"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2018 The OpenXLA Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class HloProfilePrinterData(google.protobuf.message.Message):
    """Describes how to pretty-print a profile counter array gathered for a specific
    HloModule.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class HloInstructionInfo(google.protobuf.message.Message):
        """Pretty-printer information about an HloInstruction."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LONG_NAME_FIELD_NUMBER: builtins.int
        SHORT_NAME_FIELD_NUMBER: builtins.int
        CATEGORY_FIELD_NUMBER: builtins.int
        FLOP_COUNT_FIELD_NUMBER: builtins.int
        TRANSCENDENTAL_COUNT_FIELD_NUMBER: builtins.int
        BYTES_ACCESSED_FIELD_NUMBER: builtins.int
        OPTIMAL_SECONDS_FIELD_NUMBER: builtins.int
        PROFILE_INDEX_FIELD_NUMBER: builtins.int
        long_name: builtins.str
        short_name: builtins.str
        category: builtins.str
        flop_count: builtins.float
        """Metrics computed by HloCostAnalysis."""
        transcendental_count: builtins.float
        bytes_accessed: builtins.int
        optimal_seconds: builtins.float
        profile_index: builtins.int
        """The index into the profile counters array for the HloInstruction
        corresponding to this HloInstructionInfo.
        """
        def __init__(
            self,
            *,
            long_name: builtins.str | None = ...,
            short_name: builtins.str | None = ...,
            category: builtins.str | None = ...,
            flop_count: builtins.float | None = ...,
            transcendental_count: builtins.float | None = ...,
            bytes_accessed: builtins.int | None = ...,
            optimal_seconds: builtins.float | None = ...,
            profile_index: builtins.int | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "bytes_accessed",
                b"bytes_accessed",
                "category",
                b"category",
                "flop_count",
                b"flop_count",
                "long_name",
                b"long_name",
                "optimal_seconds",
                b"optimal_seconds",
                "profile_index",
                b"profile_index",
                "short_name",
                b"short_name",
                "transcendental_count",
                b"transcendental_count",
            ],
        ) -> None: ...

    @typing.final
    class HloComputationInfo(google.protobuf.message.Message):
        """Pretty-printer information about an HloComputation."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        PROFILE_INDEX_FIELD_NUMBER: builtins.int
        INSTRUCTION_INFOS_FIELD_NUMBER: builtins.int
        name: builtins.str
        profile_index: builtins.int
        """The index into the profile counters array for the HloComputation
        corresponding to this HloComputationInfo.
        """
        @property
        def instruction_infos(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
            global___HloProfilePrinterData.HloInstructionInfo
        ]:
            """HloInstructionInfos for every HloInstruction in the HloComputation for
            corresponding to this HloComputattionInfo.
            """

        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            profile_index: builtins.int | None = ...,
            instruction_infos: collections.abc.Iterable[global___HloProfilePrinterData.HloInstructionInfo] | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "instruction_infos", b"instruction_infos", "name", b"name", "profile_index", b"profile_index"
            ],
        ) -> None: ...

    @typing.final
    class ExtraMetricsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.int
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.int | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    COMPUTATION_INFOS_FIELD_NUMBER: builtins.int
    PROFILE_COUNTERS_SIZE_FIELD_NUMBER: builtins.int
    EXTRA_METRICS_FIELD_NUMBER: builtins.int
    ENTRY_COMPUTATION_FIELD_NUMBER: builtins.int
    profile_counters_size: builtins.int
    """The size of the profile counters array we will pretty-print."""
    entry_computation: builtins.str
    """Name of the entry computation."""
    @property
    def computation_infos(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HloProfilePrinterData.HloComputationInfo]:
        """HloComputationInfos for every HloComputation in the HloModule."""

    @property
    def extra_metrics(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.int]:
        """Maps extra metric name to the index into the profile counters array."""

    def __init__(
        self,
        *,
        computation_infos: collections.abc.Iterable[global___HloProfilePrinterData.HloComputationInfo] | None = ...,
        profile_counters_size: builtins.int | None = ...,
        extra_metrics: collections.abc.Mapping[builtins.str, builtins.int] | None = ...,
        entry_computation: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "computation_infos",
            b"computation_infos",
            "entry_computation",
            b"entry_computation",
            "extra_metrics",
            b"extra_metrics",
            "profile_counters_size",
            b"profile_counters_size",
        ],
    ) -> None: ...

global___HloProfilePrinterData = HloProfilePrinterData



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class GrpcPayloadContainer(google.protobuf.message.Message):
    """Used to serialize and transmit tensorflow::Status payloads through
    grpc::Status `error_details` since grpc::Status lacks payload API.
    TODO(b/204231601): Use GRPC API once supported.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class PayloadsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.bytes
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.bytes | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    PAYLOADS_FIELD_NUMBER: builtins.int
    @property
    def payloads(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.bytes]: ...
    def __init__(self, *, payloads: collections.abc.Mapping[builtins.str, builtins.bytes] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["payloads", b"payloads"]) -> None: ...

global___GrpcPayloadContainer = GrpcPayloadContainer

@typing.final
class GrpcPayloadsLost(google.protobuf.message.Message):
    """If included as a payload, this message flags the Status to have lost payloads
    during the GRPC transmission.
    URI: "type.googleapis.com/tensorflow.distributed_runtime.GrpcPayloadsLost"
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___GrpcPayloadsLost = GrpcPayloadsLost

@typing.final
class WorkerPossiblyRestarted(google.protobuf.message.Message):
    """If included as a payload, this message flags the Status to be a possible
    outcome of a worker restart.
    URI:
    "type.googleapis.com/tensorflow.distributed_runtime.WorkerPossiblyRestarted"
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___WorkerPossiblyRestarted = WorkerPossiblyRestarted



from braintree.amex_express_checkout_card import AmexExpressCheckoutCard
from braintree.android_pay_card import AndroidPayCard
from braintree.apple_pay_card import ApplePayCard
from braintree.credit_card import CreditCard
from braintree.europe_bank_account import EuropeBankAccount
from braintree.masterpass_card import MasterpassCard
from braintree.paypal_account import PayPalAccount
from braintree.samsung_pay_card import SamsungPayCard
from braintree.sepa_direct_debit_account import SepaDirectDebitAccount
from braintree.unknown_payment_method import UnknownPaymentMethod
from braintree.us_bank_account import UsBankAccount
from braintree.venmo_account import VenmoAccount
from braintree.visa_checkout_card import VisaCheckoutCard

def parse_payment_method(
    gateway, attributes
) -> (
    PayPalAccount
    | CreditCard
    | EuropeBankAccount
    | ApplePayCard
    | AndroidPayCard
    | AmexExpressCheckoutCard
    | SepaDirectDebitAccount
    | VenmoAccount
    | UsBankAccount
    | VisaCheckoutCard
    | MasterpassCard
    | SamsungPayCard
    | UnknownPaymentMethod
): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..cmd import Command

class clean(Command):
    description: str
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    build_base: Incomplete
    build_lib: Incomplete
    build_temp: Incomplete
    build_scripts: Incomplete
    bdist_base: Incomplete
    all: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...



from collections.abc import Callable
from typing import Any, overload

from tensorflow import Tensor

class Constraint:
    def get_config(self) -> dict[str, Any]: ...
    def __call__(self, w: Tensor, /) -> Tensor: ...

@overload
def get(identifier: None) -> None: ...
@overload
def get(identifier: str | dict[str, Any] | Constraint) -> Constraint: ...
@overload
def get(identifier: Callable[[Tensor], Tensor]) -> Callable[[Tensor], Tensor]: ...
def __getattr__(name: str): ...  # incomplete module



from typing import Final

class ErrorCodes:
    class Address:
        CannotBeBlank: Final = "81801"
        CompanyIsInvalid: Final = "91821"
        CompanyIsTooLong: Final = "81802"
        CountryCodeAlpha2IsNotAccepted: Final = "91814"
        CountryCodeAlpha3IsNotAccepted: Final = "91816"
        CountryCodeNumericIsNotAccepted: Final = "91817"
        CountryNameIsNotAccepted: Final = "91803"
        ExtendedAddressIsInvalid: Final = "91823"
        ExtendedAddressIsTooLong: Final = "81804"
        FirstNameIsInvalid: Final = "91819"
        FirstNameIsTooLong: Final = "81805"
        InconsistentCountry: Final = "91815"
        IsInvalid: Final = "91828"
        LastNameIsInvalid: Final = "91820"
        LastNameIsTooLong: Final = "81806"
        LocalityIsInvalid: Final = "91824"
        LocalityIsTooLong: Final = "81807"
        PostalCodeInvalidCharacters: Final = "81813"
        PostalCodeIsInvalid: Final = "91826"
        PostalCodeIsRequired: Final = "81808"
        PostalCodeIsRequiredForCardBrandAndProcessor: Final = "81828"
        PostalCodeIsTooLong: Final = "81809"
        RegionIsInvalid: Final = "91825"
        RegionIsTooLong: Final = "81810"
        StateIsInvalidForSellerProtection: Final = "81827"
        StreetAddressIsInvalid: Final = "91822"
        StreetAddressIsRequired: Final = "81811"
        StreetAddressIsTooLong: Final = "81812"
        TooManyAddressesPerCustomer: Final = "91818"

    class AndroidPay:
        AndroidPayCardsAreNotAccepted: Final = "83708"

    class ApplePay:
        ApplePayCardsAreNotAccepted: Final = "83501"
        CustomerIdIsRequiredForVaulting: Final = "83502"
        TokenIsInUse: Final = "93503"
        PaymentMethodNonceConsumed: Final = "93504"
        PaymentMethodNonceUnknown: Final = "93505"
        PaymentMethodNonceLocked: Final = "93506"
        PaymentMethodNonceCardTypeIsNotAccepted: Final = "83518"
        CannotUpdateApplePayCardUsingPaymentMethodNonce: Final = "93507"
        NumberIsRequired: Final = "93508"
        ExpirationMonthIsRequired: Final = "93509"
        ExpirationYearIsRequired: Final = "93510"
        CryptogramIsRequired: Final = "93511"
        DecryptionFailed: Final = "83512"
        Disabled: Final = "93513"
        MerchantNotConfigured: Final = "93514"
        MerchantKeysAlreadyConfigured: Final = "93515"
        MerchantKeysNotConfigured: Final = "93516"
        CertificateInvalid: Final = "93517"
        CertificateMismatch: Final = "93519"
        InvalidToken: Final = "83520"
        PrivateKeyMismatch: Final = "93521"
        KeyMismatchStoringCertificate: Final = "93522"
        CustomerIdIsInvalid: Final = "93528"
        BillingAddressFormatIsInvalid: Final = "93529"

    class AuthorizationFingerprint:
        MissingFingerprint: Final = "93201"
        InvalidFormat: Final = "93202"
        SignatureRevoked: Final = "93203"
        InvalidCreatedAt: Final = "93204"
        InvalidPublicKey: Final = "93205"
        InvalidSignature: Final = "93206"
        OptionsNotAllowedWithoutCustomer: Final = "93207"

    class ClientToken:
        CustomerDoesNotExist: Final = "92804"
        FailOnDuplicatePaymentMethodRequiresCustomerId: Final = "92803"
        FailOnDuplicatePaymentMethodForCustomerRequiresCustomerId: Final = "92805"
        InvalidDomainFormat: Final = "92011"
        MakeDefaultRequiresCustomerId: Final = "92801"
        MerchantAccountDoesNotExist: Final = "92807"
        ProxyMerchantDoesNotExist: Final = "92805"
        TooManyDomains: Final = "92810"
        UnsupportedVersion: Final = "92806"
        VerifyCardRequiresCustomerId: Final = "92802"

    class CreditCard:
        BillingAddressConflict: Final = "91701"
        BillingAddressFormatIsInvalid: Final = "91744"
        BillingAddressIdIsInvalid: Final = "91702"
        CannotUpdateCardUsingPaymentMethodNonce: Final = "91735"
        CardholderNameIsTooLong: Final = "81723"
        CreditCardTypeIsNotAccepted: Final = "81703"
        CreditCardTypeIsNotAcceptedBySubscriptionMerchantAccount: Final = "81718"
        CustomerIdIsInvalid: Final = "91705"
        CustomerIdIsRequired: Final = "91704"
        CvvIsInvalid: Final = "81707"
        CvvIsRequired: Final = "81706"
        CvvVerificationFailed: Final = "81736"
        DuplicateCardExists: Final = "81724"
        DuplicateCardExistsForCustomer: Final = "81763"
        ExpirationDateConflict: Final = "91708"
        ExpirationDateIsInvalid: Final = "81710"
        ExpirationDateIsRequired: Final = "81709"
        ExpirationDateYearIsInvalid: Final = "81711"
        ExpirationMonthIsInvalid: Final = "81712"
        ExpirationYearIsInvalid: Final = "81713"
        InvalidParamsForCreditCardUpdate: Final = "91745"
        InvalidVenmoSDKPaymentMethodCode: Final = "91727"
        LimitExceededforDuplicatePaymentMethodCheckForCustomer: Final = "81764"
        NetworkTokenizationAttributeCryptogramIsRequired: Final = "81762"
        NumberHasInvalidLength: Final = "81716"
        NumberLengthIsInvalid: Final = "81716"
        NumberIsInvalid: Final = "81715"
        NumberIsProhibited: Final = "81750"
        NumberIsRequired: Final = "81714"
        NumberMustBeTestNumber: Final = "81717"
        PaymentMethodConflict: Final = "81725"
        PaymentMethodIsNotACreditCard: Final = "91738"
        PaymentMethodNonceCardTypeIsNotAccepted: Final = "91734"
        PaymentMethodNonceConsumed: Final = "91731"
        PaymentMethodNonceLocked: Final = "91733"
        PaymentMethodNonceUnknown: Final = "91732"
        PostalCodeVerificationFailed: Final = "81737"
        TokenInvalid: Final = "91718"
        TokenFormatIsInvalid: Final = "91718"
        TokenIsInUse: Final = "91719"
        TokenIsNotAllowed: Final = "91721"
        TokenIsRequired: Final = "91722"
        TokenIsTooLong: Final = "91720"
        VenmoSDKPaymentMethodCodeCardTypeIsNotAccepted: Final = "91726"
        VerificationNotSupportedOnThisMerchantAccount: Final = "91730"
        VerificationAccountTypeIsInvald: Final = "91757"
        VerificationAccountTypeNotSupported: Final = "91758"

        class Options:
            UpdateExistingTokenIsInvalid: Final = "91723"
            UpdateExistingTokenNotAllowed: Final = "91729"
            VerificationAmountCannotBeNegative: Final = "91739"
            VerificationAmountFormatIsInvalid: Final = "91740"
            VerificationAmountIsTooLarge: Final = "91752"
            VerificationAmountNotSupportedByProcessor: Final = "91741"
            VerificationMerchantAccountIdIsInvalid: Final = "91728"
            VerificationMerchantAccountIsForbidden: Final = "91743"
            VerificationMerchantAccountIsSuspended: Final = "91742"

    class Customer:
        CompanyIsTooLong: Final = "81601"
        CustomFieldIsInvalid: Final = "91602"
        CustomFieldIsTooLong: Final = "81603"
        EmailIsInvalid: Final = "81604"
        EmailFormatIsInvalid: Final = "81604"
        EmailIsRequired: Final = "81606"
        EmailIsTooLong: Final = "81605"
        FaxIsTooLong: Final = "81607"
        FirstNameIsTooLong: Final = "81608"
        IdIsInUse: Final = "91609"
        IdIsInvalid: Final = "91610"
        IdIsNotAllowed: Final = "91611"
        IdIsRequired: Final = "91613"
        IdIsTooLong: Final = "91612"
        LastNameIsTooLong: Final = "81613"
        PhoneIsTooLong: Final = "81614"
        VaultedPaymentInstrumentNonceBelongsToDifferentCustomer: Final = "91617"
        WebsiteIsInvalid: Final = "81616"
        WebsiteFormatIsInvalid: Final = "81616"
        WebsiteIsTooLong: Final = "81615"

    class Descriptor:
        DynamicDescriptorsDisabled: Final = "92203"
        InternationalNameFormatIsInvalid: Final = "92204"
        InternationalPhoneFormatIsInvalid: Final = "92205"
        NameFormatIsInvalid: Final = "92201"
        PhoneFormatIsInvalid: Final = "92202"
        UrlFormatIsInvalid: Final = "92206"

    class Dispute:
        CanOnlyAddEvidenceToOpenDispute: Final = "95701"
        CanOnlyRemoveEvidenceFromOpenDispute: Final = "95702"
        CanOnlyAddEvidenceDocumentToDispute: Final = "95703"
        CanOnlyAcceptOpenDispute: Final = "95704"
        CanOnlyFinalizeOpenDispute: Final = "95705"
        CanOnlyCreateEvidenceWithValidCategory: Final = "95706"
        EvidenceContentDateInvalid: Final = "95707"
        EvidenceContentTooLong: Final = "95708"
        EvidenceContentARNTooLong: Final = "95709"
        EvidenceContentPhoneTooLong: Final = "95710"
        EvidenceCategoryTextOnly: Final = "95711"
        EvidenceCategoryDocumentOnly: Final = "95712"
        EvidenceCategoryNotForReasonCode: Final = "95713"
        EvidenceCategoryDuplicate: Final = "95714"
        EvidenceContentEmailInvalid: Final = "95715"
        DigitalGoodsMissingEvidence: Final = "95720"
        DigitalGoodsMissingDownloadDate: Final = "95721"
        NonDisputedPriorTransactionEvidenceMissingARN: Final = "95722"
        NonDisputedPriorTransactionEvidenceMissingDate: Final = "95723"
        RecurringTransactionEvidenceMissingDate: Final = "95724"
        RecurringTransactionEvidenceMissingARN: Final = "95725"
        ValidEvidenceRequiredToFinalize: Final = "95726"

    class DocumentUpload:
        KindIsInvalid: Final = "84901"
        FileIsTooLarge: Final = "84902"
        FileTypeIsInvalid: Final = "84903"
        FileIsMalformedOrEncrypted: Final = "84904"
        FileIsTooLong: Final = "84905"
        FileIsEmpty: Final = "84906"

    class Merchant:
        CountryCannotBeBlank: Final = "83603"
        CountryCodeAlpha2IsInvalid: Final = "93607"
        CountryCodeAlpha2IsNotAccepted: Final = "93606"
        CountryCodeAlpha3IsInvalid: Final = "93605"
        CountryCodeAlpha3IsNotAccepted: Final = "93604"
        CountryCodeNumericIsInvalid: Final = "93609"
        CountryCodeNumericIsNotAccepted: Final = "93608"
        CountryNameIsInvalid: Final = "93611"
        CountryNameIsNotAccepted: Final = "93610"
        CurrenciesAreInvalid: Final = "93614"
        EmailFormatIsInvalid: Final = "93602"
        EmailIsRequired: Final = "83601"
        InconsistentCountry: Final = "93612"
        PaymentMethodsAreInvalid: Final = "93613"
        PaymentMethodsAreNotAllowed: Final = "93615"
        MerchantAccountExistsForCurrency: Final = "93616"
        CurrencyIsRequired: Final = "93617"
        CurrencyIsInvalid: Final = "93618"
        NoMerchantAccounts: Final = "93619"
        MerchantAccountExistsForId: Final = "93620"

    class MerchantAccount:
        class ApplicantDetails:
            Declined: Final = "82626"
            DeclinedMasterCardMatch: Final = "82622"
            DeclinedOFAC: Final = "82621"
            DeclinedFailedKYC: Final = "82623"
            DeclinedSsnInvalid: Final = "82624"
            DeclinedSsnMatchesDeceased: Final = "82625"

    class OAuth:
        InvalidGrant: Final = "93801"
        InvalidCredentials: Final = "93802"
        InvalidScope: Final = "93803"
        InvalidRequest: Final = "93804"
        UnsupportedGrantType: Final = "93805"

    class Verification:
        ThreeDSecureAuthenticationIdIsInvalid: Final = "942196"
        ThreeDSecureAuthenticationIdDoesntMatchNonceThreeDSecureAuthentication: Final = "942198"
        ThreeDSecureTransactionPaymentMethodDoesntMatchThreeDSecureAuthenticationPaymentMethod: Final = "942197"
        ThreeDSecureAuthenticationIdWithThreeDSecurePassThruIsInvalid: Final = "942199"
        ThreeDSecureAuthenticationFailed: Final = "94271"
        ThreeDSecureTokenIsInvalid: Final = "94268"
        ThreeDSecureVerificationDataDoesntMatchVerify: Final = "94270"
        MerchantAccountDoesNotSupport3DSecure: Final = "942169"
        MerchantAcountDoesNotMatch3DSecureMerchantAccount: Final = "94284"
        AmountDoesNotMatch3DSecureAmount: Final = "94285"

        class ThreeDSecurePassThru:
            EciFlagIsRequired: Final = "942113"
            EciFlagIsInvalid: Final = "942114"
            CavvIsRequired: Final = "942116"
            ThreeDSecureVersionIsRequired: Final = "942117"
            ThreeDSecureVersionIsInvalid: Final = "942119"
            AuthenticationResponseIsInvalid: Final = "942120"
            DirectoryResponseIsInvalid: Final = "942121"
            CavvAlgorithmIsInvalid: Final = "942122"

        class Options:
            AmountCannotBeNegative: Final = "94201"
            AmountFormatIsInvalid: Final = "94202"
            AmountIsTooLarge: Final = "94207"
            AmountNotSupportedByProcessor: Final = "94203"
            MerchantAccountIdIsInvalid: Final = "94204"
            MerchantAccountIsSuspended: Final = "94205"
            MerchantAccountIsForbidden: Final = "94206"
            AccountTypeIsInvalid: Final = "942184"
            AccountTypeNotSupported: Final = "942185"

    class PaymentMethod:
        CannotForwardPaymentMethodType: Final = "93106"
        PaymentMethodParamsAreRequired: Final = "93101"
        NonceIsInvalid: Final = "93102"
        NonceIsRequired: Final = "93103"
        CustomerIdIsRequired: Final = "93104"
        CustomerIdIsInvalid: Final = "93105"
        PaymentMethodNonceConsumed: Final = "93107"
        PaymentMethodNonceUnknown: Final = "93108"
        PaymentMethodNonceLocked: Final = "93109"
        PaymentMethodNoLongerSupported: Final = "93117"
        AuthExpired: Final = "92911"
        CannotHaveFundingSourceWithoutAccessToken: Final = "92912"
        InvalidFundingSourceSelection: Final = "92913"
        CannotUpdatePayPalAccountUsingPaymentMethodNonce: Final = "92914"

        class Options:
            UsBankAccountVerificationMethodIsInvalid: Final = "93121"

    class PayPalAccount:
        CannotHaveBothAccessTokenAndConsentCode: Final = "82903"
        CannotVaultOneTimeUsePayPalAccount: Final = "82902"
        ConsentCodeOrAccessTokenIsRequired: Final = "82901"
        CustomerIdIsRequiredForVaulting: Final = "82905"
        InvalidParamsForPayPalAccountUpdate: Final = "92915"
        PayPalAccountsAreNotAccepted: Final = "82904"
        PayPalCommunicationError: Final = "92910"
        PaymentMethodNonceConsumed: Final = "92907"
        PaymentMethodNonceLocked: Final = "92909"
        PaymentMethodNonceUnknown: Final = "92908"
        TokenIsInUse: Final = "92906"

    class PayPalPaymentResource:
        NonceExpired: Final = "97301"
        IdNotSupported: Final = "97302"
        NonceRequired: Final = "97303"
        InvalidEmail: Final = "97304"
        EmailTooLong: Final = "97305"
        ExpectedLineItemCollection: Final = "97306"
        ExpectedLineItemHash: Final = "97307"
        ExpectedLineItemDebit: Final = "97308"
        InvalidUnitAmount: Final = "97309"
        InvalidUnitTaxAmount: Final = "97310"
        IsoCodeRequired: Final = "97311"
        IsoCodeUnsupported: Final = "97312"
        ShippingFieldsMissing: Final = "97313"
        InvalidAmountBreakdown: Final = "97314"
        ExpectedShippingOptionCollection: Final = "97315"
        ShippingOptionsRequired: Final = "97316"
        ShippingOptionFieldsMissing: Final = "97317"
        InvalidShippingOptionType: Final = "97318"
        ShippingOptionIdReused: Final = "97319"
        TooManyShippingOptionsSelected: Final = "97320"
        ShippingOptionMustMatchBreakdown: Final = "97321"
        LineItemsShouldMatchTotal: Final = "97322"
        LineItemsTaxShouldMatchTotal: Final = "97323"
        PatchCallFailed: Final = "97324"
        InvalidAmount: Final = "97325"
        ShippingIdTooLong: Final = "97326"
        ShippingLabelTooLong: Final = "97327"
        ShippingFullNameTooLong: Final = "97328"
        ShippingAddressTooLong: Final = "97329"
        ShippingExtendedAddressTooLong: Final = "97330"
        ShippingLocalityTooLong: Final = "97331"
        ShippingRegionTooLong: Final = "97332"
        CountryCodeTooLong: Final = "97333"
        NationalNumberTooLong: Final = "97334"
        PostalCodeTooLong: Final = "97335"
        DescriptionTooLong: Final = "97336"
        CustomFieldTooLong: Final = "97337"
        OrderIdTooLong: Final = "97338"

    class SettlementBatchSummary:
        CustomFieldIsInvalid: Final = "82303"
        SettlementDateIsInvalid: Final = "82302"
        SettlementDateIsRequired: Final = "82301"

    class SEPAMandate:
        TypeIsRequired: Final = "93304"
        IBANInvalidCharacter: Final = "83305"
        BICInvalidCharacter: Final = "83306"
        BICLengthIsInvalid: Final = "83307"
        BICUnsupportedCountry: Final = "83308"
        IBANUnsupportedCountry: Final = "83309"
        IBANInvalidFormat: Final = "83310"
        BillingAddressConflict: Final = "93311"
        BillingAddressIdIsInvalid: Final = "93312"
        TypeIsInvalid: Final = "93313"

    class EuropeBankAccount:
        BICIsRequired: Final = "83302"
        IBANIsRequired: Final = "83303"
        AccountHolderNameIsRequired: Final = "83301"

    class SepaDirectDebitAccount:
        SepaDebitAccountPaymentMethodMandateTypeIsNotSupported: Final = "87115"
        SepaDebitAccountPaymentMethodCustomerIdIsInvalid: Final = "87116"
        SepaDebitAccountPaymentMethodCustomerIdIsRequired: Final = "87117"

    class Subscription:
        BillingDayOfMonthCannotBeUpdated: Final = "91918"
        BillingDayOfMonthIsInvalid: Final = "91914"
        BillingDayOfMonthMustBeNumeric: Final = "91913"
        CannotAddDuplicateAddonOrDiscount: Final = "91911"
        CannotEditCanceledSubscription: Final = "81901"
        CannotEditExpiredSubscription: Final = "81910"
        CannotEditPriceChangingFieldsOnPastDueSubscription: Final = "91920"
        FirstBillingDateCannotBeInThePast: Final = "91916"
        FirstBillingDateCannotBeUpdated: Final = "91919"
        FirstBillingDateIsInvalid: Final = "91915"
        IdIsInUse: Final = "81902"
        InconsistentNumberOfBillingCycles: Final = "91908"
        InconsistentStartDate: Final = "91917"
        InvalidRequestFormat: Final = "91921"
        MerchantAccountDoesNotSupportInstrumentType: Final = "91930"
        MerchantAccountIdIsInvalid: Final = "91901"
        MismatchCurrencyISOCode: Final = "91923"
        NumberOfBillingCyclesCannotBeBlank: Final = "91912"
        NumberOfBillingCyclesIsTooSmall: Final = "91909"
        NumberOfBillingCyclesMustBeGreaterThanZero: Final = "91907"
        NumberOfBillingCyclesMustBeNumeric: Final = "91906"
        PaymentMethodNonceCardTypeIsNotAccepted: Final = "91924"
        PaymentMethodNonceInstrumentTypeDoesNotSupportSubscriptions: Final = "91929"
        PaymentMethodNonceIsInvalid: Final = "91925"
        PaymentMethodNonceNotAssociatedWithCustomer: Final = "91926"
        PaymentMethodNonceUnvaultedCardIsNotAccepted: Final = "91927"
        PaymentMethodTokenCardTypeIsNotAccepted: Final = "91902"
        PaymentMethodTokenInstrumentTypeDoesNotSupportSubscriptions: Final = "91928"
        PaymentMethodTokenIsInvalid: Final = "91903"
        PaymentMethodTokenNotAssociatedWithCustomer: Final = "91905"
        PlanBillingFrequencyCannotBeUpdated: Final = "91922"
        PlanIdIsInvalid: Final = "91904"
        PriceCannotBeBlank: Final = "81903"
        PriceFormatIsInvalid: Final = "81904"
        PriceIsTooLarge: Final = "81923"
        StatusIsCanceled: Final = "81905"
        TokenFormatIsInvalid: Final = "81906"
        TrialDurationFormatIsInvalid: Final = "81907"
        TrialDurationIsRequired: Final = "81908"
        TrialDurationUnitIsInvalid: Final = "81909"

        class Modification:
            AmountCannotBeBlank: Final = "92003"
            AmountIsInvalid: Final = "92002"
            AmountIsTooLarge: Final = "92023"
            CannotEditModificationsOnPastDueSubscription: Final = "92022"
            CannotUpdateAndRemove: Final = "92015"
            ExistingIdIsIncorrectKind: Final = "92020"
            ExistingIdIsInvalid: Final = "92011"
            ExistingIdIsRequired: Final = "92012"
            IdToRemoveIsIncorrectKind: Final = "92021"
            IdToRemoveIsNotPresent: Final = "92016"
            InconsistentNumberOfBillingCycles: Final = "92018"
            InheritedFromIdIsInvalid: Final = "92013"
            InheritedFromIdIsRequired: Final = "92014"
            Missing: Final = "92024"
            NumberOfBillingCyclesCannotBeBlank: Final = "92017"
            NumberOfBillingCyclesIsInvalid: Final = "92005"
            NumberOfBillingCyclesMustBeGreaterThanZero: Final = "92019"
            QuantityCannotBeBlank: Final = "92004"
            QuantityIsInvalid: Final = "92001"
            QuantityMustBeGreaterThanZero: Final = "92010"
            IdToRemoveIsInvalid: Final = "92025"

    class Transaction:
        AdjustmentAmountMustBeGreaterThanZero: Final = "95605"
        AmountCannotBeNegative: Final = "81501"
        AmountDoesNotMatch3DSecureAmount: Final = "91585"
        AmountIsInvalid: Final = "81503"
        AmountFormatIsInvalid: Final = "81503"
        AmountIsRequired: Final = "81502"
        AmountIsTooLarge: Final = "81528"
        AmountMustBeGreaterThanZero: Final = "81531"
        AmountNotSupportedByProcessor: Final = "815193"
        BillingAddressConflict: Final = "91530"
        BillingPhoneNumberIsInvalid: Final = "915206"
        CannotBeVoided: Final = "91504"
        CannotCancelRelease: Final = "91562"
        CannotCloneCredit: Final = "91543"
        CannotCloneMarketplaceTransaction: Final = "915137"
        CannotCloneTransactionWithPayPalAccount: Final = "91573"
        CannotCloneTransactionWithVaultCreditCard: Final = "91540"
        CannotCloneUnsuccessfulTransaction: Final = "91542"
        CannotCloneVoiceAuthorizations: Final = "91541"
        CannotRefundCredit: Final = "91505"
        CannotRefundSettlingTransaction: Final = "91574"
        CannotRefundUnlessSettled: Final = "91506"
        CannotRefundWithPendingMerchantAccount: Final = "91559"
        CannotRefundWithSuspendedMerchantAccount: Final = "91538"
        CannotSimulateTransactionSettlement: Final = "91575"
        CannotSubmitForPartialSettlement: Final = "915103"
        CannotSubmitForSettlement: Final = "91507"
        CannotUpdateTransactionDetailsNotSubmittedForSettlement: Final = "915129"
        ChannelIsTooLong: Final = "91550"
        CreditCardIsRequired: Final = "91508"
        CustomFieldIsInvalid: Final = "91526"
        CustomFieldIsTooLong: Final = "81527"
        CustomerDefaultPaymentMethodCardTypeIsNotAccepted: Final = "81509"
        CustomerDoesNotHaveCreditCard: Final = "91511"
        CustomerIdIsInvalid: Final = "91510"
        DiscountAmountCannotBeNegative: Final = "915160"
        DiscountAmountFormatIsInvalid: Final = "915159"
        DiscountAmountIsTooLarge: Final = "915161"
        ExchangeRateQuoteIdIsTooLong: Final = "915229"
        FailedAuthAdjustmentAllowRetry: Final = "95603"
        FailedAuthAdjustmentHardDecline: Final = "95602"
        FinalAuthSubmitForSettlementForDifferentAmount: Final = "95601"
        HasAlreadyBeenRefunded: Final = "91512"
        LineItemsExpected: Final = "915158"
        MerchantAccountDoesNotMatch3DSecureMerchantAccount: Final = "91584"
        MerchantAccountDoesNotSupportMOTO: Final = "91558"
        MerchantAccountDoesNotSupportRefunds: Final = "91547"
        MerchantAccountIdDoesNotMatchSubscription: Final = "915180"
        MerchantAccountIdIsInvalid: Final = "91513"
        MerchantAccountIsSuspended: Final = "91514"
        NoNetAmountToPerformAuthAdjustment: Final = "95606"
        OrderIdIsTooLong: Final = "91501"
        PayPalAuthExpired: Final = "91579"
        PayPalNotEnabled: Final = "91576"
        PayPalVaultRecordMissingData: Final = "91583"
        PaymentInstrumentNotSupportedByMerchantAccount: Final = "91577"
        PaymentInstrumentTypeIsNotAccepted: Final = "915101"
        PaymentInstrumentWithExternalVaultIsInvalid: Final = "915176"
        PaymentMethodConflict: Final = "91515"
        PaymentMethodConflictWithVenmoSDK: Final = "91549"
        PaymentMethodDoesNotBelongToCustomer: Final = "91516"
        PaymentMethodDoesNotBelongToSubscription: Final = "91527"
        PaymentMethodNonceCardTypeIsNotAccepted: Final = "91567"
        PaymentMethodNonceConsumed: Final = "91564"
        PaymentMethodNonceHasNoValidPaymentInstrumentType: Final = "91569"
        PaymentMethodNonceLocked: Final = "91566"
        PaymentMethodNonceUnknown: Final = "91565"
        PaymentMethodTokenCardTypeIsNotAccepted: Final = "91517"
        PaymentMethodTokenIsInvalid: Final = "91518"
        ProcessingMerchantCategoryCodeIsInvalid: Final = "915265"
        ProcessorAuthorizationCodeCannotBeSet: Final = "91519"
        ProcessorAuthorizationCodeIsInvalid: Final = "81520"
        ProcessorDoesNotSupportAuths: Final = "915104"
        ProcessorDoesNotSupportAuthAdjustment: Final = "915222"
        ProcessorDoesNotSupportCredits: Final = "91546"
        ProcessorDoesNotSupportIncrementalAuth: Final = "915220"
        ProcessorDoesNotSupportMotoForCardType: Final = "915195"
        ProcessorDoesNotSupportPartialAuthReversal: Final = "915221"
        ProcessorDoesNotSupportPartialSettlement: Final = "915102"
        ProcessorDoesNotSupportUpdatingDescriptor: Final = "915108"
        ProcessorDoesNotSupportUpdatingOrderId: Final = "915107"
        ProcessorDoesNotSupportUpdatingTransactionDetails: Final = "915130"
        ProcessorDoesNotSupportVoiceAuthorizations: Final = "91545"
        ProductSkuIsInvalid: Final = "915202"
        PurchaseOrderNumberIsInvalid: Final = "91548"
        PurchaseOrderNumberIsTooLong: Final = "91537"
        RefundAmountIsTooLarge: Final = "91521"
        RefundAuthHardDeclined: Final = "915200"
        RefundAuthSoftDeclined: Final = "915201"
        ScaExemptionInvalid: Final = "915213"
        ServiceFeeAmountCannotBeNegative: Final = "91554"
        ServiceFeeAmountFormatIsInvalid: Final = "91555"
        ServiceFeeAmountIsTooLarge: Final = "91556"
        ServiceFeeAmountNotAllowedOnMasterMerchantAccount: Final = "91557"
        ServiceFeeIsNotAllowedOnCredits: Final = "91552"
        ServiceFeeNotAcceptedForPayPal: Final = "91578"
        SettlementAmountIsLessThanServiceFeeAmount: Final = "91551"
        SettlementAmountIsTooLarge: Final = "91522"
        ShippingAddressDoesntMatchCustomer: Final = "91581"
        ShippingAmountCannotBeNegative: Final = "915163"
        ShippingAmountFormatIsInvalid: Final = "915162"
        ShippingAmountIsTooLarge: Final = "915164"
        ShippingMethodIsInvalid: Final = "915203"
        ShippingPhoneNumberIsInvalid: Final = "915204"
        ShipsFromPostalCodeInvalidCharacters: Final = "915167"
        ShipsFromPostalCodeIsInvalid: Final = "915166"
        ShipsFromPostalCodeIsTooLong: Final = "915165"
        SubscriptionDoesNotBelongToCustomer: Final = "91529"
        SubscriptionIdIsInvalid: Final = "91528"
        SubscriptionStatusMustBePastDue: Final = "91531"
        TaxAmountCannotBeNegative: Final = "81534"
        TaxAmountFormatIsInvalid: Final = "81535"
        TaxAmountIsRequiredForAibSwedish: Final = "815224"
        TaxAmountIsTooLarge: Final = "81536"
        ThreeDSecureAuthenticationFailed: Final = "81571"
        ThreeDSecureAuthenticationIdDoesntMatchNonceThreeDSecureAuthentication: Final = "915198"
        ThreeDSecureAuthenticationIdIsInvalid: Final = "915196"
        ThreeDSecureAuthenticationIdWithThreeDSecurePassThruIsInvalid: Final = "915199"
        ThreeDSecureAuthenticationResponseIsInvalid: Final = "915120"
        ThreeDSecureCavvAlgorithmIsInvalid: Final = "915122"
        ThreeDSecureCavvIsRequired: Final = "915116"
        ThreeDSecureDirectoryResponseIsInvalid: Final = "915121"
        ThreeDSecureEciFlagIsInvalid: Final = "915114"
        ThreeDSecureEciFlagIsRequired: Final = "915113"
        ThreeDSecureMerchantAccountDoesNotSupportCardType: Final = "915131"
        ThreeDSecureTokenIsInvalid: Final = "91568"
        ThreeDSecureTransactionDataDoesntMatchVerify: Final = "91570"
        ThreeDSecureTransactionPaymentMethodDoesntMatchThreeDSecureAuthenticationPaymentMethod: Final = "915197"
        ThreeDSecureXidIsRequired: Final = "915115"
        TooManyLineItems: Final = "915157"
        TransactionIsNotEligibleForAdjustment: Final = "915219"
        TransactionMustBeInStateAuthorized: Final = "915218"
        TransactionSourceIsInvalid: Final = "915133"
        TransferDetailsAreNotApplicableForThisMerchantAccount: Final = "97511"
        TransferDetailsAreRequired: Final = "97510"
        TransferReceiverAccountReferenceNumberIsNotValid: Final = "97509"
        TransferReceiverFirstNameIsNotValid: Final = "97507"
        TransferReceiverLastNameIsNotValid: Final = "97508"
        TransferReceiverTaxIdIsNotValid: Final = "97506"
        TransferSenderAccountReferenceNumberIsNotValid: Final = "97505"
        TransferSenderFirstNameIsNotValid: Final = "97503"
        TransferSenderLastNameIsNotValid: Final = "97504"
        TransferSenderTaxIdIsNotValid: Final = "97502"
        TransferTypeIsInvalid: Final = "97501"
        TypeIsInvalid: Final = "91523"
        TypeIsRequired: Final = "91524"
        UnsupportedVoiceAuthorization: Final = "91539"
        UsBankAccountNonceMustBePlaidVerified: Final = "915171"
        UsBankAccountNotVerified: Final = "915172"

        class ExternalVault:
            StatusIsInvalid: Final = "915175"
            StatusWithPreviousNetworkTransactionIdIsInvalid: Final = "915177"
            CardTypeIsInvalid: Final = "915178"
            PreviousNetworkTransactionIdIsInvalid: Final = "915179"

        class Options:
            SubmitForSettlementIsRequiredForCloning: Final = "91544"
            SubmitForSettlementIsRequiredForPayPalUnilateral: Final = "91582"
            UseBillingForShippingDisabled: Final = "91572"
            VaultIsDisabled: Final = "91525"

            class PayPal:
                CustomFieldTooLong: Final = "91580"

            class CreditCard:
                AccountTypeIsInvalid: Final = "915184"
                AccountTypeNotSupported: Final = "915185"
                AccountTypeDebitDoesNotSupportAuths: Final = "915186"

        class Industry:
            IndustryTypeIsInvalid: Final = "93401"

            class Lodging:
                EmptyData: Final = "93402"
                FolioNumberIsInvalid: Final = "93403"
                CheckInDateIsInvalid: Final = "93404"
                CheckOutDateIsInvalid: Final = "93405"
                CheckOutDateMustFollowCheckInDate: Final = "93406"
                UnknownDataField: Final = "93407"
                RoomRateMustBeGreaterThanZero: Final = "93433"
                RoomRateFormatIsInvalid: Final = "93434"
                RoomRateIsTooLarge: Final = "93435"
                RoomTaxMustBeGreaterThanZero: Final = "93436"
                RoomTaxFormatIsInvalid: Final = "93437"
                RoomTaxIsTooLarge: Final = "93438"
                NoShowIndicatorIsInvalid: Final = "93439"
                AdvancedDepositIndicatorIsInvalid: Final = "93440"
                FireSafetyIndicatorIsInvalid: Final = "93441"
                PropertyPhoneIsInvalid: Final = "93442"

            class TravelCruise:
                EmptyData: Final = "93408"
                UnknownDataField: Final = "93409"
                TravelPackageIsInvalid: Final = "93410"
                DepartureDateIsInvalid: Final = "93411"
                LodgingCheckInDateIsInvalid: Final = "93412"
                LodgingCheckOutDateIsInvalid: Final = "93413"

            class TravelFlight:
                EmptyData: Final = "93414"
                UnknownDataField: Final = "93415"
                CustomerCodeIsTooLong: Final = "93416"
                FareAmountCannotBeNegative: Final = "93417"
                FareAmountFormatIsInvalid: Final = "93418"
                FareAmountIsTooLarge: Final = "93419"
                FeeAmountCannotBeNegative: Final = "93420"
                FeeAmountFormatIsInvalid: Final = "93421"
                FeeAmountIsTooLarge: Final = "93422"
                IssuedDateFormatIsInvalid: Final = "93423"
                IssuingCarrierCodeIsTooLong: Final = "93424"
                PassengerMiddleInitialIsTooLong: Final = "93425"
                RestrictedTicketIsRequired: Final = "93426"
                TaxAmountCannotBeNegative: Final = "93427"
                TaxAmountFormatIsInvalid: Final = "93428"
                TaxAmountIsTooLarge: Final = "93429"
                TicketNumberIsTooLong: Final = "93430"
                LegsExpected: Final = "93431"
                TooManyLegs: Final = "93432"

            class Leg:
                class TravelFlight:
                    ArrivalAirportCodeIsTooLong: Final = "96301"
                    ArrivalTimeFormatIsInvalid: Final = "96302"
                    CarrierCodeIsTooLong: Final = "96303"
                    ConjunctionTicketIsTooLong: Final = "96304"
                    CouponNumberIsTooLong: Final = "96305"
                    DepartureAirportCodeIsTooLong: Final = "96306"
                    DepartureTimeFormatIsInvalid: Final = "96307"
                    ExchangeTicketIsTooLong: Final = "96308"
                    FareAmountCannotBeNegative: Final = "96309"
                    FareAmountFormatIsInvalid: Final = "96310"
                    FareAmountIsTooLarge: Final = "96311"
                    FareBasisCodeIsTooLong: Final = "96312"
                    FeeAmountCannotBeNegative: Final = "96313"
                    FeeAmountFormatIsInvalid: Final = "96314"
                    FeeAmountIsTooLarge: Final = "96315"
                    ServiceClassIsTooLong: Final = "96316"
                    TaxAmountCannotBeNegative: Final = "96317"
                    TaxAmountFormatIsInvalid: Final = "96318"
                    TaxAmountIsTooLarge: Final = "96319"
                    TicketNumberIsTooLong: Final = "96320"

            class AdditionalCharge:
                KindIsInvalid: Final = "96601"
                KindMustBeUnique: Final = "96602"
                AmountMustBeGreaterThanZero: Final = "96603"
                AmountFormatIsInvalid: Final = "96604"
                AmountIsTooLarge: Final = "96605"
                AmountIsRequired: Final = "96606"

        class LineItem:
            CommodityCodeIsTooLong: Final = "95801"
            DescriptionIsTooLong: Final = "95803"
            DiscountAmountCannotBeNegative: Final = "95806"
            DiscountAmountFormatIsInvalid: Final = "95804"
            DiscountAmountIsTooLarge: Final = "95805"
            KindIsInvalid: Final = "95807"
            KindIsRequired: Final = "95808"
            NameIsRequired: Final = "95822"
            NameIsTooLong: Final = "95823"
            ProductCodeIsTooLong: Final = "95809"
            QuantityFormatIsInvalid: Final = "95810"
            QuantityIsRequired: Final = "95811"
            QuantityIsTooLarge: Final = "95812"
            TaxAmountCannotBeNegative: Final = "95829"
            TaxAmountFormatIsInvalid: Final = "95827"
            TaxAmountIsTooLarge: Final = "95828"
            TotalAmountFormatIsInvalid: Final = "95813"
            TotalAmountIsRequired: Final = "95814"
            TotalAmountIsTooLarge: Final = "95815"
            TotalAmountMustBeGreaterThanZero: Final = "95816"
            UPCCodeIsMissing: Final = "95830"
            UPCCodeIsTooLong: Final = "95831"
            UPCTypeIsInvalid: Final = "95833"
            UPCTypeIsMissing: Final = "95832"
            UnitAmountFormatIsInvalid: Final = "95817"
            UnitAmountIsRequired: Final = "95818"
            UnitAmountIsTooLarge: Final = "95819"
            UnitAmountMustBeGreaterThanZero: Final = "95820"
            UnitOfMeasureIsTooLarge: Final = "95821"
            UnitTaxAmountCannotBeNegative: Final = "95826"
            UnitTaxAmountFormatIsInvalid: Final = "95824"
            UnitTaxAmountIsTooLarge: Final = "95825"

    class UsBankAccountVerification:
        NotConfirmable: Final = "96101"
        MustBeMicroTransfersVerification: Final = "96102"
        AmountsDoNotMatch: Final = "96103"
        TooManyConfirmationAttempts: Final = "96104"
        UnableToConfirmDepositAmounts: Final = "96105"
        InvalidDepositAmounts: Final = "96106"

    class RiskData:
        CustomerBrowserIsTooLong: Final = "94701"
        CustomerDeviceIdIsTooLong: Final = "94702"
        CustomerLocationZipInvalidCharacters: Final = "94703"
        CustomerLocationZipIsInvalid: Final = "94704"
        CustomerLocationZipIsTooLong: Final = "94705"
        CustomerTenureIsTooLong: Final = "94706"



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class UsersByEmail:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def search_users_by_email(
        self, email: str, fields: list[str] | None = None, include_fields: bool = True
    ) -> list[dict[str, Incomplete]]: ...
    async def search_users_by_email_async(
        self, email: str, fields: list[str] | None = None, include_fields: bool = True
    ) -> list[dict[str, Incomplete]]: ...



class DeviceCredentialMixin:
    def get_client_id(self) -> None: ...
    def get_scope(self) -> None: ...
    def get_user_code(self) -> None: ...
    def is_expired(self) -> None: ...

class DeviceCredentialDict(dict[str, object], DeviceCredentialMixin):
    def get_client_id(self): ...
    def get_scope(self): ...
    def get_user_code(self): ...
    def get_nonce(self): ...
    def get_auth_time(self): ...
    def is_expired(self): ...



from _typeshed import Incomplete

from win32com.server.util import ListEnumeratorGateway

class EnumDebugCodeContexts(ListEnumeratorGateway): ...
class EnumDebugStackFrames(ListEnumeratorGateway): ...
class EnumDebugApplicationNodes(ListEnumeratorGateway): ...
class EnumRemoteDebugApplications(ListEnumeratorGateway): ...
class EnumRemoteDebugApplicationThreads(ListEnumeratorGateway): ...

class DebugDocumentInfo:
    def GetName(self, dnt) -> None: ...
    def GetDocumentClassId(self) -> None: ...

class DebugDocumentProvider(DebugDocumentInfo):
    def GetDocument(self) -> None: ...

class DebugApplicationNode(DebugDocumentProvider):
    def EnumChildren(self) -> None: ...
    def GetParent(self) -> None: ...
    def SetDocumentProvider(self, pddp) -> None: ...
    def Close(self) -> None: ...
    def Attach(self, parent) -> None: ...
    def Detach(self) -> None: ...

class DebugApplicationNodeEvents:
    def onAddChild(self, child) -> None: ...
    def onRemoveChild(self, child) -> None: ...
    def onDetach(self) -> None: ...
    def onAttach(self, parent) -> None: ...

class DebugDocument(DebugDocumentInfo): ...

class DebugDocumentText(DebugDocument):
    def GetDocumentAttributes(self) -> None: ...
    def GetSize(self) -> None: ...
    def GetPositionOfLine(self, cLineNumber) -> None: ...
    def GetLineOfPosition(self, charPos) -> None: ...
    def GetText(self, charPos, maxChars, wantAttr) -> None: ...
    def GetPositionOfContext(self, debugDocumentContext) -> None: ...
    def GetContextOfPosition(self, charPos, maxChars) -> None: ...

class DebugDocumentTextExternalAuthor:
    def GetPathName(self) -> None: ...
    def GetFileName(self) -> None: ...
    def NotifyChanged(self) -> None: ...

class DebugDocumentTextEvents:
    def onDestroy(self) -> None: ...
    def onInsertText(self, cCharacterPosition, cNumToInsert) -> None: ...
    def onRemoveText(self, cCharacterPosition, cNumToRemove) -> None: ...
    def onReplaceText(self, cCharacterPosition, cNumToReplace) -> None: ...
    def onUpdateTextAttributes(self, cCharacterPosition, cNumToUpdate) -> None: ...
    def onUpdateDocumentAttributes(self, textdocattr) -> None: ...

class DebugDocumentContext:
    def GetDocument(self) -> None: ...
    def EnumCodeContexts(self) -> None: ...

class DebugCodeContext:
    def GetDocumentContext(self) -> None: ...
    def SetBreakPoint(self, bps) -> None: ...

class DebugStackFrame:
    def GetCodeContext(self) -> None: ...
    def GetDescriptionString(self, fLong) -> None: ...
    def GetLanguageString(self) -> None: ...
    def GetThread(self) -> None: ...
    def GetDebugProperty(self) -> None: ...

class DebugDocumentHost:
    def GetDeferredText(self, dwTextStartCookie, maxChars, bWantAttr) -> None: ...
    def GetScriptTextAttributes(self, codeText, delimterText, flags) -> None: ...
    def OnCreateDocumentContext(self) -> None: ...
    def GetPathName(self) -> None: ...
    def GetFileName(self) -> None: ...
    def NotifyChanged(self) -> None: ...

class DebugDocumentTextConnectServer:
    cookieNo: int
    connections: Incomplete
    def EnumConnections(self) -> None: ...
    def GetConnectionInterface(self) -> None: ...
    def GetConnectionPointContainer(self): ...
    def Advise(self, pUnk): ...
    def Unadvise(self, cookie): ...
    def EnumConnectionPoints(self) -> None: ...
    def FindConnectionPoint(self, iid): ...

class RemoteDebugApplicationEvents:
    def OnConnectDebugger(self, appDebugger) -> None: ...
    def OnDisconnectDebugger(self) -> None: ...
    def OnSetName(self, name) -> None: ...
    def OnDebugOutput(self, string) -> None: ...
    def OnClose(self) -> None: ...
    def OnEnterBreakPoint(self, rdat) -> None: ...
    def OnLeaveBreakPoint(self, rdat) -> None: ...
    def OnCreateThread(self, rdat) -> None: ...
    def OnDestroyThread(self, rdat) -> None: ...
    def OnBreakFlagChange(self, abf, rdat) -> None: ...

class DebugExpressionContext:
    def ParseLanguageText(self, code, radix, delim, flags) -> None: ...
    def GetLanguageInfo(self) -> None: ...

class DebugExpression:
    def Start(self, callback) -> None: ...
    def Abort(self) -> None: ...
    def QueryIsComplete(self) -> None: ...
    def GetResultAsString(self) -> None: ...
    def GetResultAsDebugProperty(self) -> None: ...

class ProvideExpressionContexts:
    def EnumExpressionContexts(self) -> None: ...



from __future__ import annotations

from typing import Tuple
from typing_extensions import assert_type


# Empty tuples, see #8275
class TupleSub(Tuple[int, ...]):
    pass


assert_type(TupleSub(), TupleSub)
assert_type(TupleSub([1, 2, 3]), TupleSub)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
TODO(b/247876220): Change package and java_package once we figure out how to
migrate.
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Code:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CodeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Code.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    OK: _Code.ValueType  # 0
    """Not an error; returned on success"""
    CANCELLED: _Code.ValueType  # 1
    """The operation was cancelled (typically by the caller)."""
    UNKNOWN: _Code.ValueType  # 2
    """Unknown error.  An example of where this error may be returned is
    if a Status value received from another address space belongs to
    an error-space that is not known in this address space.  Also
    errors raised by APIs that do not return enough error information
    may be converted to this error.
    """
    INVALID_ARGUMENT: _Code.ValueType  # 3
    """Client specified an invalid argument.  Note that this differs
    from FAILED_PRECONDITION.  INVALID_ARGUMENT indicates arguments
    that are problematic regardless of the state of the system
    (e.g., a malformed file name).
    """
    DEADLINE_EXCEEDED: _Code.ValueType  # 4
    """Deadline expired before operation could complete.  For operations
    that change the state of the system, this error may be returned
    even if the operation has completed successfully.  For example, a
    successful response from a server could have been delayed long
    enough for the deadline to expire.
    """
    NOT_FOUND: _Code.ValueType  # 5
    """Some requested entity (e.g., file or directory) was not found.
    For privacy reasons, this code *may* be returned when the client
    does not have the access right to the entity.
    """
    ALREADY_EXISTS: _Code.ValueType  # 6
    """Some entity that we attempted to create (e.g., file or directory)
    already exists.
    """
    PERMISSION_DENIED: _Code.ValueType  # 7
    """The caller does not have permission to execute the specified
    operation.  PERMISSION_DENIED must not be used for rejections
    caused by exhausting some resource (use RESOURCE_EXHAUSTED
    instead for those errors).  PERMISSION_DENIED must not be
    used if the caller can not be identified (use UNAUTHENTICATED
    instead for those errors).
    """
    UNAUTHENTICATED: _Code.ValueType  # 16
    """The request does not have valid authentication credentials for the
    operation.
    """
    RESOURCE_EXHAUSTED: _Code.ValueType  # 8
    """Some resource has been exhausted, perhaps a per-user quota, or
    perhaps the entire file system is out of space.
    """
    FAILED_PRECONDITION: _Code.ValueType  # 9
    """Operation was rejected because the system is not in a state
    required for the operation's execution.  For example, directory
    to be deleted may be non-empty, an rmdir operation is applied to
    a non-directory, etc.

    A litmus test that may help a service implementor in deciding
    between FAILED_PRECONDITION, ABORTED, and UNAVAILABLE:
     (a) Use UNAVAILABLE if the client can retry just the failing call.
     (b) Use ABORTED if the client should retry at a higher-level
         (e.g., restarting a read-modify-write sequence).
     (c) Use FAILED_PRECONDITION if the client should not retry until
         the system state has been explicitly fixed.  E.g., if an "rmdir"
         fails because the directory is non-empty, FAILED_PRECONDITION
         should be returned since the client should not retry unless
         they have first fixed up the directory by deleting files from it.
     (d) Use FAILED_PRECONDITION if the client performs conditional
         REST Get/Update/Delete on a resource and the resource on the
         server does not match the condition. E.g., conflicting
         read-modify-write on the same resource.
    """
    ABORTED: _Code.ValueType  # 10
    """The operation was aborted, typically due to a concurrency issue
    like sequencer check failures, transaction aborts, etc.

    See litmus test above for deciding between FAILED_PRECONDITION,
    ABORTED, and UNAVAILABLE.
    """
    OUT_OF_RANGE: _Code.ValueType  # 11
    """Operation tried to iterate past the valid input range.  E.g., seeking or
    reading past end of file.

    Unlike INVALID_ARGUMENT, this error indicates a problem that may
    be fixed if the system state changes. For example, a 32-bit file
    system will generate INVALID_ARGUMENT if asked to read at an
    offset that is not in the range [0,2^32-1], but it will generate
    OUT_OF_RANGE if asked to read from an offset past the current
    file size.

    There is a fair bit of overlap between FAILED_PRECONDITION and
    OUT_OF_RANGE.  We recommend using OUT_OF_RANGE (the more specific
    error) when it applies so that callers who are iterating through
    a space can easily look for an OUT_OF_RANGE error to detect when
    they are done.
    """
    UNIMPLEMENTED: _Code.ValueType  # 12
    """Operation is not implemented or not supported/enabled in this service."""
    INTERNAL: _Code.ValueType  # 13
    """Internal errors.  Means some invariant expected by the underlying
    system has been broken.  If you see one of these errors,
    something is very broken.
    """
    UNAVAILABLE: _Code.ValueType  # 14
    """The service is currently unavailable.  This is a most likely a
    transient condition and may be corrected by retrying with
    a backoff.

    See litmus test above for deciding between FAILED_PRECONDITION,
    ABORTED, and UNAVAILABLE.
    """
    DATA_LOSS: _Code.ValueType  # 15
    """Unrecoverable data loss or corruption."""
    DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_: _Code.ValueType  # 20
    """An extra enum entry to prevent people from writing code that
    fails to compile when a new code is added.

    Nobody should ever reference this enumeration entry. In particular,
    if you write C++ code that switches on this enumeration, add a default:
    case instead of a case that mentions this enumeration entry.

    Nobody should rely on the value (currently 20) listed here.  It
    may change in the future.
    """

class Code(_Code, metaclass=_CodeEnumTypeWrapper):
    """The canonical error codes for TensorFlow APIs.

    Warnings:

    -   Do not change any numeric assignments.
    -   Changes to this list should only be made if there is a compelling
        need that can't be satisfied in another way.  Such changes
        must be approved by at least two OWNERS.
    -   These error codes must match gRPC and protobuf error codes (except for
        DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_).

    Sometimes multiple error codes may apply.  Services should return
    the most specific error code that applies.  For example, prefer
    OUT_OF_RANGE over FAILED_PRECONDITION if both codes apply.
    Similarly prefer NOT_FOUND or ALREADY_EXISTS over FAILED_PRECONDITION.
    """

OK: Code.ValueType  # 0
"""Not an error; returned on success"""
CANCELLED: Code.ValueType  # 1
"""The operation was cancelled (typically by the caller)."""
UNKNOWN: Code.ValueType  # 2
"""Unknown error.  An example of where this error may be returned is
if a Status value received from another address space belongs to
an error-space that is not known in this address space.  Also
errors raised by APIs that do not return enough error information
may be converted to this error.
"""
INVALID_ARGUMENT: Code.ValueType  # 3
"""Client specified an invalid argument.  Note that this differs
from FAILED_PRECONDITION.  INVALID_ARGUMENT indicates arguments
that are problematic regardless of the state of the system
(e.g., a malformed file name).
"""
DEADLINE_EXCEEDED: Code.ValueType  # 4
"""Deadline expired before operation could complete.  For operations
that change the state of the system, this error may be returned
even if the operation has completed successfully.  For example, a
successful response from a server could have been delayed long
enough for the deadline to expire.
"""
NOT_FOUND: Code.ValueType  # 5
"""Some requested entity (e.g., file or directory) was not found.
For privacy reasons, this code *may* be returned when the client
does not have the access right to the entity.
"""
ALREADY_EXISTS: Code.ValueType  # 6
"""Some entity that we attempted to create (e.g., file or directory)
already exists.
"""
PERMISSION_DENIED: Code.ValueType  # 7
"""The caller does not have permission to execute the specified
operation.  PERMISSION_DENIED must not be used for rejections
caused by exhausting some resource (use RESOURCE_EXHAUSTED
instead for those errors).  PERMISSION_DENIED must not be
used if the caller can not be identified (use UNAUTHENTICATED
instead for those errors).
"""
UNAUTHENTICATED: Code.ValueType  # 16
"""The request does not have valid authentication credentials for the
operation.
"""
RESOURCE_EXHAUSTED: Code.ValueType  # 8
"""Some resource has been exhausted, perhaps a per-user quota, or
perhaps the entire file system is out of space.
"""
FAILED_PRECONDITION: Code.ValueType  # 9
"""Operation was rejected because the system is not in a state
required for the operation's execution.  For example, directory
to be deleted may be non-empty, an rmdir operation is applied to
a non-directory, etc.

A litmus test that may help a service implementor in deciding
between FAILED_PRECONDITION, ABORTED, and UNAVAILABLE:
 (a) Use UNAVAILABLE if the client can retry just the failing call.
 (b) Use ABORTED if the client should retry at a higher-level
     (e.g., restarting a read-modify-write sequence).
 (c) Use FAILED_PRECONDITION if the client should not retry until
     the system state has been explicitly fixed.  E.g., if an "rmdir"
     fails because the directory is non-empty, FAILED_PRECONDITION
     should be returned since the client should not retry unless
     they have first fixed up the directory by deleting files from it.
 (d) Use FAILED_PRECONDITION if the client performs conditional
     REST Get/Update/Delete on a resource and the resource on the
     server does not match the condition. E.g., conflicting
     read-modify-write on the same resource.
"""
ABORTED: Code.ValueType  # 10
"""The operation was aborted, typically due to a concurrency issue
like sequencer check failures, transaction aborts, etc.

See litmus test above for deciding between FAILED_PRECONDITION,
ABORTED, and UNAVAILABLE.
"""
OUT_OF_RANGE: Code.ValueType  # 11
"""Operation tried to iterate past the valid input range.  E.g., seeking or
reading past end of file.

Unlike INVALID_ARGUMENT, this error indicates a problem that may
be fixed if the system state changes. For example, a 32-bit file
system will generate INVALID_ARGUMENT if asked to read at an
offset that is not in the range [0,2^32-1], but it will generate
OUT_OF_RANGE if asked to read from an offset past the current
file size.

There is a fair bit of overlap between FAILED_PRECONDITION and
OUT_OF_RANGE.  We recommend using OUT_OF_RANGE (the more specific
error) when it applies so that callers who are iterating through
a space can easily look for an OUT_OF_RANGE error to detect when
they are done.
"""
UNIMPLEMENTED: Code.ValueType  # 12
"""Operation is not implemented or not supported/enabled in this service."""
INTERNAL: Code.ValueType  # 13
"""Internal errors.  Means some invariant expected by the underlying
system has been broken.  If you see one of these errors,
something is very broken.
"""
UNAVAILABLE: Code.ValueType  # 14
"""The service is currently unavailable.  This is a most likely a
transient condition and may be corrected by retrying with
a backoff.

See litmus test above for deciding between FAILED_PRECONDITION,
ABORTED, and UNAVAILABLE.
"""
DATA_LOSS: Code.ValueType  # 15
"""Unrecoverable data loss or corruption."""
DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_: Code.ValueType  # 20
"""An extra enum entry to prevent people from writing code that
fails to compile when a new code is added.

Nobody should ever reference this enumeration entry. In particular,
if you write C++ code that switches on this enumeration, add a default:
case instead of a case that mentions this enumeration entry.

Nobody should rely on the value (currently 20) listed here.  It
may change in the future.
"""
global___Code = Code



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Literal

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["average_degree_connectivity"]

@_dispatchable
def average_degree_connectivity(
    G: Graph[_Node],
    source: Literal["in+out", "out", "in"] = "in+out",
    target: Literal["in+out", "out", "in"] = "in+out",
    nodes: Iterable[Incomplete] | None = None,
    weight: str | None = None,
) -> dict[Incomplete, int | float]: ...



from typing import ClassVar, Final

from authlib.jose.rfc7517 import AsymmetricKey
from cryptography.hazmat.primitives.asymmetric.ed448 import Ed448PrivateKey, Ed448PublicKey
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey, Ed25519PublicKey
from cryptography.hazmat.primitives.asymmetric.x448 import X448PrivateKey, X448PublicKey
from cryptography.hazmat.primitives.asymmetric.x25519 import X25519PrivateKey, X25519PublicKey

PUBLIC_KEYS_MAP: Final[dict[str, type]]
PRIVATE_KEYS_MAP: Final[dict[str, type]]

class OKPKey(AsymmetricKey):
    kty: str
    REQUIRED_JSON_FIELDS: ClassVar[list[str]]
    PUBLIC_KEY_FIELDS = REQUIRED_JSON_FIELDS
    PRIVATE_KEY_FIELDS: ClassVar[list[str]]
    PUBLIC_KEY_CLS: ClassVar[tuple[type, ...]]
    PRIVATE_KEY_CLS: ClassVar[tuple[type, ...]]
    SSH_PUBLIC_PREFIX: ClassVar[bytes]
    def exchange_shared_key(self, pubkey): ...
    @staticmethod
    def get_key_curve(key): ...
    def load_private_key(self) -> Ed25519PrivateKey | Ed448PrivateKey | X25519PrivateKey | X448PrivateKey: ...
    def load_public_key(self) -> Ed25519PublicKey | Ed448PublicKey | X25519PublicKey | X448PublicKey: ...
    def dumps_private_key(self) -> dict[str, str | None]: ...
    def dumps_public_key(self, public_key=None) -> dict[str, str | None]: ...
    @classmethod
    def generate_key(cls, crv: str = "Ed25519", options=None, is_private: bool = False) -> OKPKey: ...



import sys
from collections.abc import Callable
from typing import Any, ClassVar, NamedTuple, type_check_only
from typing_extensions import TypeAlias

__all__ = ["scheduler"]

_ActionCallback: TypeAlias = Callable[..., Any]

if sys.version_info >= (3, 10):
    class Event(NamedTuple):
        time: float
        priority: Any
        sequence: int
        action: _ActionCallback
        argument: tuple[Any, ...]
        kwargs: dict[str, Any]

else:
    @type_check_only
    class _EventBase(NamedTuple):
        time: float
        priority: Any
        action: _ActionCallback
        argument: tuple[Any, ...]
        kwargs: dict[str, Any]

    class Event(_EventBase):
        __hash__: ClassVar[None]  # type: ignore[assignment]

class scheduler:
    timefunc: Callable[[], float]
    delayfunc: Callable[[float], object]

    def __init__(self, timefunc: Callable[[], float] = ..., delayfunc: Callable[[float], object] = ...) -> None: ...
    def enterabs(
        self, time: float, priority: Any, action: _ActionCallback, argument: tuple[Any, ...] = (), kwargs: dict[str, Any] = ...
    ) -> Event: ...
    def enter(
        self, delay: float, priority: Any, action: _ActionCallback, argument: tuple[Any, ...] = (), kwargs: dict[str, Any] = ...
    ) -> Event: ...
    def run(self, blocking: bool = True) -> float | None: ...
    def cancel(self, event: Event) -> None: ...
    def empty(self) -> bool: ...
    @property
    def queue(self) -> list[Event]: ...



import os
from _typeshed import Incomplete
from typing import Final

from reportlab.pdfbase.pdfdoc import PDFObject
from reportlab.platypus.flowables import Flowable

__version__: Final[str]

def xorKey(num, key): ...

CLOBBERID: int
CLOBBERPERMISSIONS: int
DEBUG: Incomplete
reserved1: int
reserved2: Incomplete
printable: Incomplete
modifiable: Incomplete
copypastable: Incomplete
annotatable: Incomplete
higherbits: int

os_urandom = os.urandom

class StandardEncryption:
    prepared: int
    userPassword: Incomplete
    ownerPassword: Incomplete
    revision: int
    canPrint: Incomplete
    canModify: Incomplete
    canCopy: Incomplete
    canAnnotate: Incomplete
    O: Incomplete
    def __init__(
        self,
        userPassword,
        ownerPassword=None,
        canPrint: int = 1,
        canModify: int = 1,
        canCopy: int = 1,
        canAnnotate: int = 1,
        strength=None,
    ) -> None: ...
    def setAllPermissions(self, value) -> None: ...
    def permissionBits(self): ...
    def encode(self, t): ...
    P: Incomplete
    key: Incomplete
    U: Incomplete
    UE: Incomplete
    OE: Incomplete
    Perms: Incomplete
    objnum: Incomplete
    def prepare(self, document, overrideID=None) -> None: ...
    version: Incomplete
    def register(self, objnum, version) -> None: ...
    def info(self): ...

class StandardEncryptionDictionary(PDFObject):
    __RefOnly__: int
    revision: Incomplete
    def __init__(self, O, OE, U, UE, P, Perms, revision) -> None: ...
    def format(self, document): ...

padding: str

def hexText(text): ...
def unHexText(hexText): ...

PadString: Incomplete

def checkRevision(revision): ...
def encryptionkey(password, OwnerKey, Permissions, FileId1, revision=None): ...
def computeO(userPassword, ownerPassword, revision): ...
def computeU(
    encryptionkey,
    encodestring=b"(\xbfN^Nu\x8aAd\x00NV\xff\xfa\x01\x08..\x00\xb6\xd0h>\x80/\x0c\xa9\xfedSiz",
    revision=None,
    documentId=None,
): ...
def checkU(encryptionkey, U) -> None: ...
def encodePDF(key, objectNumber, generationNumber, string, revision=None): ...
def equalityCheck(observed, expected, label) -> None: ...
def test() -> None: ...
def encryptCanvas(
    canvas,
    userPassword,
    ownerPassword=None,
    canPrint: int = 1,
    canModify: int = 1,
    canCopy: int = 1,
    canAnnotate: int = 1,
    strength: int = 40,
) -> None: ...

class EncryptionFlowable(StandardEncryption, Flowable):
    def wrap(self, availWidth, availHeight): ...
    def draw(self) -> None: ...

def encryptDocTemplate(
    dt,
    userPassword,
    ownerPassword=None,
    canPrint: int = 1,
    canModify: int = 1,
    canCopy: int = 1,
    canAnnotate: int = 1,
    strength: int = 40,
) -> None: ...
def encryptPdfInMemory(
    inputPDF,
    userPassword,
    ownerPassword=None,
    canPrint: int = 1,
    canModify: int = 1,
    canCopy: int = 1,
    canAnnotate: int = 1,
    strength: int = 40,
): ...
def encryptPdfOnDisk(
    inputFileName,
    outputFileName,
    userPassword,
    ownerPassword=None,
    canPrint: int = 1,
    canModify: int = 1,
    canCopy: int = 1,
    canAnnotate: int = 1,
    strength: int = 40,
): ...
def scriptInterp() -> None: ...
def main() -> None: ...



from re import Pattern
from typing import Final

RE_SOURCE_FILENAME: Pattern[str]
RE_TARGET_FILENAME: Pattern[str]
RE_DIFF_GIT_HEADER: Pattern[str]
RE_DIFF_GIT_HEADER_URI_LIKE: Pattern[str]
RE_DIFF_GIT_HEADER_NO_PREFIX: Pattern[str]
RE_DIFF_GIT_DELETED_FILE: Pattern[str]
RE_DIFF_GIT_NEW_FILE: Pattern[str]
RE_HUNK_HEADER: Pattern[str]
RE_HUNK_BODY_LINE: Pattern[str]
RE_HUNK_EMPTY_BODY_LINE: Pattern[str]
RE_NO_NEWLINE_MARKER: Pattern[str]
RE_BINARY_DIFF: Pattern[str]

DEFAULT_ENCODING: Final = "UTF-8"
DEV_NULL: Final = "/dev/null"
LINE_TYPE_ADDED: Final = "+"
LINE_TYPE_REMOVED: Final = "-"
LINE_TYPE_CONTEXT: Final = " "
LINE_TYPE_EMPTY: Final = ""
LINE_TYPE_NO_NEWLINE: Final = "\\"
LINE_VALUE_NO_NEWLINE: Final = " No newline at end of file"



from typing import Any, overload

from braintree.graphql.unions.customer_recommendations import CustomerRecommendations

class CustomerRecommendationsPayload:
    session_id: str
    is_in_paypal_network: bool
    recommendations: CustomerRecommendations
    @overload
    def __init__(
        self,
        session_id: None = None,
        is_in_paypal_network: None = None,
        recommendations: None = None,
        *,
        response: dict[str, Any],
    ): ...
    @overload
    def __init__(
        self, session_id: str, is_in_paypal_network: bool, recommendations: CustomerRecommendations, response: None = None
    ): ...



from _collections_abc import dict_values
from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["label_propagation_communities", "asyn_lpa_communities", "fast_label_propagation_communities"]

@_dispatchable
def fast_label_propagation_communities(G: Graph[_Node], *, weight=None, seed=None) -> Generator[Incomplete]: ...
@_dispatchable
def asyn_lpa_communities(
    G: Graph[_Node], weight: str | None = None, seed: int | RandomState | None = None
) -> Generator[Incomplete, Incomplete, None]: ...
@_dispatchable
def label_propagation_communities(G: Graph[_Node]) -> dict_values[Incomplete, set[Incomplete]]: ...



import re
from _typeshed import Incomplete
from collections.abc import Callable
from types import ModuleType

tag_regexp: re.Pattern[str]

def getETreeBuilder(ElementTreeImplementation, fullTree: bool = False) -> dict[str, Incomplete]: ...

getETreeModule: Callable[..., ModuleType]



from email.headerregistry import Address
from email.message import EmailMessage
from typing_extensions import assert_type

msg = EmailMessage()
msg["To"] = "receiver@example.com"
msg["From"] = Address("Sender Name", "sender", "example.com")

for a in msg.iter_attachments():
    assert_type(a, EmailMessage)



from _typeshed import Incomplete

from authlib.oauth2.rfc6750 import BearerTokenGenerator

class JWTBearerTokenGenerator(BearerTokenGenerator):
    issuer: Incomplete
    alg: Incomplete
    def __init__(self, issuer, alg: str = "RS256", refresh_token_generator=None, expires_generator=None) -> None: ...
    def get_jwks(self) -> None: ...
    def get_extra_claims(self, client, grant_type, user, scope): ...
    def get_audiences(self, client, user, scope) -> str | list[str]: ...
    def get_acr(self, user) -> str | None: ...
    def get_auth_time(self, user) -> int | None: ...
    def get_amr(self, user) -> list[str] | None: ...
    def get_jti(self, client, grant_type, user, scope) -> str: ...
    # Override seems safe, but mypy doesn't like that it's a callabe protocol in the base
    def access_token_generator(self, client, grant_type, user, scope): ...  # type: ignore[override]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class StructuredValue(google.protobuf.message.Message):
    """`StructuredValue` represents a dynamically typed value representing various
    data structures that are inspired by Python data structures typically used in
    TensorFlow functions as inputs and outputs.

    For example when saving a Layer there may be a `training` argument. If the
    user passes a boolean True/False, that switches between two concrete
    TensorFlow functions. In order to switch between them in the same way after
    loading the SavedModel, we need to represent "True" and "False".

    A more advanced example might be a function which takes a list of
    dictionaries mapping from strings to Tensors. In order to map from
    user-specified arguments `[{"a": tf.constant(1.)}, {"q": tf.constant(3.)}]`
    after load to the right saved TensorFlow function, we need to represent the
    nested structure and the strings, recording that we have a trace for anything
    matching `[{"a": tf.TensorSpec(None, tf.float32)}, {"q": tf.TensorSpec([],
    tf.float64)}]` as an example.

    Likewise functions may return nested structures of Tensors, for example
    returning a dictionary mapping from strings to Tensors. In order for the
    loaded function to return the same structure we need to serialize it.

    This is an ergonomic aid for working with loaded SavedModels, not a promise
    to serialize all possible function signatures. For example we do not expect
    to pickle generic Python objects, and ideally we'd stay language-agnostic.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NONE_VALUE_FIELD_NUMBER: builtins.int
    FLOAT64_VALUE_FIELD_NUMBER: builtins.int
    INT64_VALUE_FIELD_NUMBER: builtins.int
    STRING_VALUE_FIELD_NUMBER: builtins.int
    BOOL_VALUE_FIELD_NUMBER: builtins.int
    TENSOR_SHAPE_VALUE_FIELD_NUMBER: builtins.int
    TENSOR_DTYPE_VALUE_FIELD_NUMBER: builtins.int
    TENSOR_SPEC_VALUE_FIELD_NUMBER: builtins.int
    TYPE_SPEC_VALUE_FIELD_NUMBER: builtins.int
    BOUNDED_TENSOR_SPEC_VALUE_FIELD_NUMBER: builtins.int
    LIST_VALUE_FIELD_NUMBER: builtins.int
    TUPLE_VALUE_FIELD_NUMBER: builtins.int
    DICT_VALUE_FIELD_NUMBER: builtins.int
    NAMED_TUPLE_VALUE_FIELD_NUMBER: builtins.int
    TENSOR_VALUE_FIELD_NUMBER: builtins.int
    NUMPY_VALUE_FIELD_NUMBER: builtins.int
    float64_value: builtins.float
    """Represents a double-precision floating-point value (a Python `float`)."""
    int64_value: builtins.int
    """Represents a signed integer value, limited to 64 bits.
    Larger values from Python's arbitrary-precision integers are unsupported.
    """
    string_value: builtins.str
    """Represents a string of Unicode characters stored in a Python `str`.
    In Python 3, this is exactly what type `str` is.
    In Python 2, this is the UTF-8 encoding of the characters.
    For strings with ASCII characters only (as often used in TensorFlow code)
    there is effectively no difference between the language versions.
    The obsolescent `unicode` type of Python 2 is not supported here.
    """
    bool_value: builtins.bool
    """Represents a boolean value."""
    tensor_dtype_value: tensorflow.core.framework.types_pb2.DataType.ValueType
    """Represents an enum value for dtype."""
    @property
    def none_value(self) -> global___NoneValue:
        """Represents None."""

    @property
    def tensor_shape_value(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto:
        """Represents a TensorShape."""

    @property
    def tensor_spec_value(self) -> global___TensorSpecProto:
        """Represents a value for tf.TensorSpec."""

    @property
    def type_spec_value(self) -> global___TypeSpecProto:
        """Represents a value for tf.TypeSpec."""

    @property
    def bounded_tensor_spec_value(self) -> global___BoundedTensorSpecProto:
        """Represents a value for tf.BoundedTensorSpec."""

    @property
    def list_value(self) -> global___ListValue:
        """Represents a list of `Value`."""

    @property
    def tuple_value(self) -> global___TupleValue:
        """Represents a tuple of `Value`."""

    @property
    def dict_value(self) -> global___DictValue:
        """Represents a dict `Value`."""

    @property
    def named_tuple_value(self) -> global___NamedTupleValue:
        """Represents Python's namedtuple."""

    @property
    def tensor_value(self) -> tensorflow.core.framework.tensor_pb2.TensorProto:
        """Represents a value for tf.Tensor."""

    @property
    def numpy_value(self) -> tensorflow.core.framework.tensor_pb2.TensorProto:
        """Represents a value for np.ndarray."""

    def __init__(
        self,
        *,
        none_value: global___NoneValue | None = ...,
        float64_value: builtins.float | None = ...,
        int64_value: builtins.int | None = ...,
        string_value: builtins.str | None = ...,
        bool_value: builtins.bool | None = ...,
        tensor_shape_value: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        tensor_dtype_value: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        tensor_spec_value: global___TensorSpecProto | None = ...,
        type_spec_value: global___TypeSpecProto | None = ...,
        bounded_tensor_spec_value: global___BoundedTensorSpecProto | None = ...,
        list_value: global___ListValue | None = ...,
        tuple_value: global___TupleValue | None = ...,
        dict_value: global___DictValue | None = ...,
        named_tuple_value: global___NamedTupleValue | None = ...,
        tensor_value: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
        numpy_value: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "bool_value",
            b"bool_value",
            "bounded_tensor_spec_value",
            b"bounded_tensor_spec_value",
            "dict_value",
            b"dict_value",
            "float64_value",
            b"float64_value",
            "int64_value",
            b"int64_value",
            "kind",
            b"kind",
            "list_value",
            b"list_value",
            "named_tuple_value",
            b"named_tuple_value",
            "none_value",
            b"none_value",
            "numpy_value",
            b"numpy_value",
            "string_value",
            b"string_value",
            "tensor_dtype_value",
            b"tensor_dtype_value",
            "tensor_shape_value",
            b"tensor_shape_value",
            "tensor_spec_value",
            b"tensor_spec_value",
            "tensor_value",
            b"tensor_value",
            "tuple_value",
            b"tuple_value",
            "type_spec_value",
            b"type_spec_value",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "bool_value",
            b"bool_value",
            "bounded_tensor_spec_value",
            b"bounded_tensor_spec_value",
            "dict_value",
            b"dict_value",
            "float64_value",
            b"float64_value",
            "int64_value",
            b"int64_value",
            "kind",
            b"kind",
            "list_value",
            b"list_value",
            "named_tuple_value",
            b"named_tuple_value",
            "none_value",
            b"none_value",
            "numpy_value",
            b"numpy_value",
            "string_value",
            b"string_value",
            "tensor_dtype_value",
            b"tensor_dtype_value",
            "tensor_shape_value",
            b"tensor_shape_value",
            "tensor_spec_value",
            b"tensor_spec_value",
            "tensor_value",
            b"tensor_value",
            "tuple_value",
            b"tuple_value",
            "type_spec_value",
            b"type_spec_value",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["kind", b"kind"]
    ) -> (
        typing.Literal[
            "none_value",
            "float64_value",
            "int64_value",
            "string_value",
            "bool_value",
            "tensor_shape_value",
            "tensor_dtype_value",
            "tensor_spec_value",
            "type_spec_value",
            "bounded_tensor_spec_value",
            "list_value",
            "tuple_value",
            "dict_value",
            "named_tuple_value",
            "tensor_value",
            "numpy_value",
        ]
        | None
    ): ...

global___StructuredValue = StructuredValue

@typing.final
class NoneValue(google.protobuf.message.Message):
    """Represents None."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___NoneValue = NoneValue

@typing.final
class ListValue(google.protobuf.message.Message):
    """Represents a Python list."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___StructuredValue]: ...
    def __init__(self, *, values: collections.abc.Iterable[global___StructuredValue] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___ListValue = ListValue

@typing.final
class TupleValue(google.protobuf.message.Message):
    """Represents a Python tuple."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___StructuredValue]: ...
    def __init__(self, *, values: collections.abc.Iterable[global___StructuredValue] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___TupleValue = TupleValue

@typing.final
class DictValue(google.protobuf.message.Message):
    """Represents a Python dict keyed by `str`.
    The comment on Unicode from Value.string_value applies analogously.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class FieldsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___StructuredValue: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___StructuredValue | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    FIELDS_FIELD_NUMBER: builtins.int
    @property
    def fields(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___StructuredValue]: ...
    def __init__(self, *, fields: collections.abc.Mapping[builtins.str, global___StructuredValue] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["fields", b"fields"]) -> None: ...

global___DictValue = DictValue

@typing.final
class PairValue(google.protobuf.message.Message):
    """Represents a (key, value) pair."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    key: builtins.str
    @property
    def value(self) -> global___StructuredValue: ...
    def __init__(self, *, key: builtins.str | None = ..., value: global___StructuredValue | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

global___PairValue = PairValue

@typing.final
class NamedTupleValue(google.protobuf.message.Message):
    """Represents Python's namedtuple."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VALUES_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PairValue]: ...
    def __init__(
        self, *, name: builtins.str | None = ..., values: collections.abc.Iterable[global___PairValue] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "values", b"values"]) -> None: ...

global___NamedTupleValue = NamedTupleValue

@typing.final
class TensorSpecProto(google.protobuf.message.Message):
    """A protobuf to represent tf.TensorSpec."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["shape", b"shape"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dtype", b"dtype", "name", b"name", "shape", b"shape"]) -> None: ...

global___TensorSpecProto = TensorSpecProto

@typing.final
class BoundedTensorSpecProto(google.protobuf.message.Message):
    """A protobuf to represent tf.BoundedTensorSpec."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    MINIMUM_FIELD_NUMBER: builtins.int
    MAXIMUM_FIELD_NUMBER: builtins.int
    name: builtins.str
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    @property
    def minimum(self) -> tensorflow.core.framework.tensor_pb2.TensorProto: ...
    @property
    def maximum(self) -> tensorflow.core.framework.tensor_pb2.TensorProto: ...
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        minimum: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
        maximum: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["maximum", b"maximum", "minimum", b"minimum", "shape", b"shape"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "dtype", b"dtype", "maximum", b"maximum", "minimum", b"minimum", "name", b"name", "shape", b"shape"
        ],
    ) -> None: ...

global___BoundedTensorSpecProto = BoundedTensorSpecProto

@typing.final
class TypeSpecProto(google.protobuf.message.Message):
    """Represents a tf.TypeSpec"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _TypeSpecClass:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeSpecClassEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[TypeSpecProto._TypeSpecClass.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: TypeSpecProto._TypeSpecClass.ValueType  # 0
        SPARSE_TENSOR_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 1
        """tf.SparseTensorSpec"""
        INDEXED_SLICES_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 2
        """tf.IndexedSlicesSpec"""
        RAGGED_TENSOR_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 3
        """tf.RaggedTensorSpec"""
        TENSOR_ARRAY_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 4
        """tf.TensorArraySpec"""
        DATA_DATASET_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 5
        """tf.data.DatasetSpec"""
        DATA_ITERATOR_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 6
        """IteratorSpec from data/ops/iterator_ops.py"""
        OPTIONAL_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 7
        """tf.OptionalSpec"""
        PER_REPLICA_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 8
        """PerReplicaSpec from distribute/values.py"""
        VARIABLE_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 9
        """tf.VariableSpec"""
        ROW_PARTITION_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 10
        """RowPartitionSpec from ragged/row_partition.py"""
        REGISTERED_TYPE_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 12
        """The type registered as type_spec_class_name."""
        EXTENSION_TYPE_SPEC: TypeSpecProto._TypeSpecClass.ValueType  # 13
        """Subclasses of tf.ExtensionType"""

    class TypeSpecClass(_TypeSpecClass, metaclass=_TypeSpecClassEnumTypeWrapper): ...
    UNKNOWN: TypeSpecProto.TypeSpecClass.ValueType  # 0
    SPARSE_TENSOR_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 1
    """tf.SparseTensorSpec"""
    INDEXED_SLICES_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 2
    """tf.IndexedSlicesSpec"""
    RAGGED_TENSOR_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 3
    """tf.RaggedTensorSpec"""
    TENSOR_ARRAY_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 4
    """tf.TensorArraySpec"""
    DATA_DATASET_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 5
    """tf.data.DatasetSpec"""
    DATA_ITERATOR_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 6
    """IteratorSpec from data/ops/iterator_ops.py"""
    OPTIONAL_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 7
    """tf.OptionalSpec"""
    PER_REPLICA_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 8
    """PerReplicaSpec from distribute/values.py"""
    VARIABLE_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 9
    """tf.VariableSpec"""
    ROW_PARTITION_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 10
    """RowPartitionSpec from ragged/row_partition.py"""
    REGISTERED_TYPE_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 12
    """The type registered as type_spec_class_name."""
    EXTENSION_TYPE_SPEC: TypeSpecProto.TypeSpecClass.ValueType  # 13
    """Subclasses of tf.ExtensionType"""

    TYPE_SPEC_CLASS_FIELD_NUMBER: builtins.int
    TYPE_STATE_FIELD_NUMBER: builtins.int
    TYPE_SPEC_CLASS_NAME_FIELD_NUMBER: builtins.int
    NUM_FLAT_COMPONENTS_FIELD_NUMBER: builtins.int
    type_spec_class: global___TypeSpecProto.TypeSpecClass.ValueType
    type_spec_class_name: builtins.str
    """The name of the TypeSpec class.
     * If type_spec_class == REGISTERED_TYPE_SPEC, the TypeSpec class is
       the one registered under this name. For types registered outside
       core TensorFlow by an add-on library, that library must be loaded
       before this value can be deserialized by nested_structure_coder.
     * If type_spec_class specifies a particular TypeSpec class, this field is
       redundant with the type_spec_class enum, and is only used for error
       reporting in older binaries that do not know the tupe_spec_class enum.
    """
    num_flat_components: builtins.int
    """The number of flat tensor components required by this TypeSpec."""
    @property
    def type_state(self) -> global___StructuredValue:
        """The value returned by TypeSpec._serialize()."""

    def __init__(
        self,
        *,
        type_spec_class: global___TypeSpecProto.TypeSpecClass.ValueType | None = ...,
        type_state: global___StructuredValue | None = ...,
        type_spec_class_name: builtins.str | None = ...,
        num_flat_components: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["type_state", b"type_state"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "num_flat_components",
            b"num_flat_components",
            "type_spec_class",
            b"type_spec_class",
            "type_spec_class_name",
            b"type_spec_class_name",
            "type_state",
            b"type_state",
        ],
    ) -> None: ...

global___TypeSpecProto = TypeSpecProto



from collections.abc import Generator
from itertools import zip_longest as zip_longest
from types import FunctionType
from typing import TypeVar

_T = TypeVar("_T")

text_type = str
string_type = str

def with_str_method(cls: _T) -> _T: ...
def with_repr_method(cls: _T) -> _T: ...
def get_methods(cls: object) -> Generator[tuple[str, FunctionType]]: ...



from typing import Any, ClassVar
from typing_extensions import Self

import passlib.utils.handlers as uh

log: Any

class sha1_crypt(uh.HasManyBackends, uh.HasRounds, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    checksum_chars: ClassVar[str]
    default_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    @classmethod
    def from_string(cls, hash: str | bytes) -> Self: ...  # type: ignore[override]
    def to_string(self, config: bool = False) -> str: ...
    backends: ClassVar[tuple[str, ...]]

__all__: list[str] = []



from collections.abc import Collection
from typing_extensions import Self

from .base import BaseMultipartGeometry
from .linestring import LineString, _ConvertibleToLineString
from .multipoint import MultiPoint

__all__ = ["MultiLineString"]

class MultiLineString(BaseMultipartGeometry[LineString]):
    __slots__: list[str] = []
    def __new__(self, lines: BaseMultipartGeometry | Collection[_ConvertibleToLineString] | None = None) -> Self: ...
    def svg(self, scale_factor: float = 1.0, stroke_color: str | None = None, opacity: float | None = None) -> str: ...  # type: ignore[override]
    # more precise base overrides
    @property
    def boundary(self) -> MultiPoint: ...



from typing import Final, Literal

J0000: Final = 1721424.5
J1970: Final = 2440587.5
JMJD: Final = 2400000.5
JULIAN_EPOCH: Final = 1721423.5
J2000: Final = 2451545.0
JULIANCENTURY: Final = 36525.0
HAVE_30_DAYS: Final = (4, 6, 9, 11)
HAVE_31_DAYS: Final = (1, 3, 5, 7, 8, 10, 12)

def leap(year: int) -> bool: ...
def month_length(year: int, month: int) -> Literal[28, 29, 30, 31]: ...
def legal_date(year: int, month: int, day: int) -> Literal[True]: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int, format_string: str = "%-d %B %y") -> str: ...



from collections.abc import Iterable
from logging import Logger
from typing import Final

log: Logger
SUPPORTED_MODULES: Final[tuple[str, ...]]
NO_DOUBLE_PATCH: Final[tuple[str, ...]]

def patch_all(double_patch: bool = False) -> None: ...
def patch(
    modules_to_patch: Iterable[str], raise_errors: bool = True, ignore_module_patterns: Iterable[str] | None = None
) -> None: ...



import abc
from _typeshed import Incomplete
from logging import Logger

from pika.adapters.utils import io_services_utils, nbio_interface

LOGGER: Logger

class AbstractSelectorIOLoop(metaclass=abc.ABCMeta):
    @property
    @abc.abstractmethod
    def READ(self): ...
    @property
    @abc.abstractmethod
    def WRITE(self): ...
    @property
    @abc.abstractmethod
    def ERROR(self): ...
    @abc.abstractmethod
    def close(self): ...
    @abc.abstractmethod
    def start(self): ...
    @abc.abstractmethod
    def stop(self): ...
    @abc.abstractmethod
    def call_later(self, delay, callback): ...
    @abc.abstractmethod
    def remove_timeout(self, timeout_handle): ...
    @abc.abstractmethod
    def add_callback(self, callback): ...
    @abc.abstractmethod
    def add_handler(self, fd, handler, events): ...
    @abc.abstractmethod
    def update_handler(self, fd, events): ...
    @abc.abstractmethod
    def remove_handler(self, fd): ...

class SelectorIOServicesAdapter(
    io_services_utils.SocketConnectionMixin,
    io_services_utils.StreamingConnectionMixin,
    nbio_interface.AbstractIOServices,
    nbio_interface.AbstractFileDescriptorServices,
):
    def __init__(self, native_loop) -> None: ...
    def get_native_ioloop(self): ...
    def close(self) -> None: ...
    def run(self) -> None: ...
    def stop(self) -> None: ...
    def add_callback_threadsafe(self, callback) -> None: ...
    def call_later(self, delay, callback): ...
    def getaddrinfo(self, host, port, on_done, family: int = 0, socktype: int = 0, proto: int = 0, flags: int = 0): ...
    def set_reader(self, fd, on_readable) -> None: ...
    def remove_reader(self, fd): ...
    def set_writer(self, fd, on_writable) -> None: ...
    def remove_writer(self, fd): ...

class _FileDescriptorCallbacks:
    __slots__ = ("reader", "writer")
    reader: Incomplete
    writer: Incomplete
    def __init__(self, reader=None, writer=None) -> None: ...

class _TimerHandle(nbio_interface.AbstractTimerReference):
    def __init__(self, handle, loop) -> None: ...
    def cancel(self) -> None: ...

class _SelectorIOLoopIOHandle(nbio_interface.AbstractIOReference):
    def __init__(self, subject) -> None: ...
    def cancel(self): ...

class _AddressResolver:
    NOT_STARTED: int
    ACTIVE: int
    CANCELED: int
    COMPLETED: int
    def __init__(self, native_loop, host, port, family, socktype, proto, flags, on_done) -> None: ...
    def start(self): ...
    def cancel(self): ...



from _hashlib import HASH
from collections.abc import Callable
from logging import Logger

from oauthlib.common import Request

log: Logger

class GrantTypeBase:
    def __getattr__(self, attr: str): ...
    def __setattr__(self, attr: str, value) -> None: ...
    def validate_authorization_request(self, request: Request): ...
    def id_token_hash(
        self, value: str, hashfunc: Callable[..., HASH] = ...  # Arguments: ReadableBuffer (string) and bool (usedforsecurity)
    ) -> str: ...
    def add_id_token(self, token, token_handler, request: Request, nonce=None): ...
    def openid_authorization_validator(self, request: Request): ...

OpenIDConnectBase = GrantTypeBase



import logging
from collections.abc import Callable, Iterator
from typing import TypeVar
from typing_extensions import ParamSpec

logger: logging.Logger

_P = ParamSpec("_P")
_R = TypeVar("_R")

class Data:
    def __init__(self, **fields: str) -> None: ...
    def __getattr__(self, key: str) -> str: ...
    def __setattr__(self, key: str, value: str) -> None: ...
    def __dir__(self) -> list[str]: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...

class Country(Data): ...
class Subdivision(Data): ...

def lazy_load(f: Callable[_P, _R]) -> Callable[_P, _R]: ...

class Database:
    data_class: type | str
    root_key: str | None
    no_index: list[str]
    filename: str
    factory: type
    objects: list[Data]
    index_names: set[str]
    indices: dict[str, dict[str, Data]]

    def __init__(self, filename: str) -> None: ...
    @lazy_load
    def add_entry(self, **kw: str) -> None: ...
    @lazy_load
    def remove_entry(self, **kw: str) -> None: ...
    @lazy_load
    def __iter__(self) -> Iterator[Data]: ...
    @lazy_load
    def __len__(self) -> int: ...
    @lazy_load
    def get(self, *, default: Data | None = ..., **kw: str) -> Data | None: ...
    @lazy_load
    def lookup(self, value: str) -> Data: ...



from typing import Any

class BlowfishEngine:
    P: Any
    S: Any
    def __init__(self) -> None: ...
    @staticmethod
    def key_to_words(data, size: int = 18): ...
    def encipher(self, l, r): ...
    def expand(self, key_words) -> None: ...
    def eks_salted_expand(self, key_words, salt_words) -> None: ...
    def eks_repeated_expand(self, key_words, salt_words, rounds) -> None: ...
    def repeat_encipher(self, l, r, count): ...

__all__ = ["BlowfishEngine"]



__all__ = ["AsyncOpenIDMixin"]

class AsyncOpenIDMixin:
    async def fetch_jwk_set(self, force: bool = False): ...
    async def userinfo(self, **kwargs): ...
    async def parse_id_token(self, token, nonce, claims_options=None, claims_cls=None, leeway: int = 120): ...



import http.client
import ssl
import sys
import urllib.request
from _typeshed import Incomplete, SupportsKeysAndGetItem
from typing import Any, TypeVar

import socks

_K = TypeVar("_K")
_V = TypeVar("_V")

def merge_dict(a: dict[_K, _V], b: SupportsKeysAndGetItem[_K, _V]) -> dict[_K, _V]: ...  # undocumented
def is_ip(s: str) -> bool: ...  # undocumented

socks4_no_rdns: set[str]  # undocumented

class SocksiPyConnection(http.client.HTTPConnection):  # undocumented
    proxyargs: tuple[int, str, int | None, bool, str | None, str | None]
    sock: socks.socksocket
    def __init__(
        self,
        proxytype: int,
        proxyaddr: str,
        proxyport: int | None = None,
        rdns: bool = True,
        username: str | None = None,
        password: str | None = None,
        host: str | None = None,
        port: int | None = None,
        timeout: float | None = ...,
        source_address: tuple[str, int] | None = None,
        blocksize: int = 8192,
    ) -> None: ...
    def connect(self) -> None: ...

class SocksiPyConnectionS(http.client.HTTPSConnection):  # undocumented
    proxyargs: tuple[int, str, int | None, bool, str | None, str | None]
    sock: socks.socksocket
    if sys.version_info >= (3, 12):
        def __init__(
            self,
            proxytype: int,
            proxyaddr: str,
            proxyport: int | None = None,
            rdns: bool = True,
            username: str | None = None,
            password: str | None = None,
            host: str | None = None,
            port: int | None = None,
            *,
            timeout: float | None = ...,
            source_address: tuple[str, int] | None = None,
            context: ssl.SSLContext | None = None,
            blocksize: int = 8192,
        ) -> None: ...
    else:
        def __init__(
            self,
            proxytype: int,
            proxyaddr: str,
            proxyport: int | None = None,
            rdns: bool = True,
            username: str | None = None,
            password: str | None = None,
            host: str | None = None,
            port: int | None = None,
            key_file: str | None = None,
            cert_file: str | None = None,
            timeout: float | None = ...,
            source_address: tuple[str, int] | None = None,
            *,
            context: ssl.SSLContext | None = None,
            check_hostname: bool | None = None,
            blocksize: int = 8192,
        ) -> None: ...

    def connect(self) -> None: ...

class SocksiPyHandler(urllib.request.HTTPHandler, urllib.request.HTTPSHandler):
    args: tuple[Incomplete, ...]  # undocumented
    kw: dict[str, Incomplete]  # undocumented
    def __init__(
        self,
        proxytype: int,
        proxyaddr: str,
        proxyport: int | None = None,
        rdns: bool = True,
        username: str | None = None,
        password: str | None = None,
        *,
        source_address: tuple[str, int] | None = None,
        blocksize: int = 8192,
        **kwargs: Any,  # any additional arguments to `SocksiPyConnection` or `SocksiPyConnectionS`
    ) -> None: ...
    def http_open(self, req: urllib.request.Request) -> http.client.HTTPResponse: ...  # undocumented
    def https_open(self, req: urllib.request.Request) -> http.client.HTTPResponse: ...  # undocumented



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.types_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class GraphTransferNodeInput(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_ID_FIELD_NUMBER: builtins.int
    OUTPUT_PORT_FIELD_NUMBER: builtins.int
    node_id: builtins.int
    output_port: builtins.int
    def __init__(self, *, node_id: builtins.int | None = ..., output_port: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["node_id", b"node_id", "output_port", b"output_port"]) -> None: ...

global___GraphTransferNodeInput = GraphTransferNodeInput

@typing.final
class GraphTransferNodeInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    NODE_ID_FIELD_NUMBER: builtins.int
    TYPE_NAME_FIELD_NUMBER: builtins.int
    SOC_OP_ID_FIELD_NUMBER: builtins.int
    PADDING_ID_FIELD_NUMBER: builtins.int
    INPUT_COUNT_FIELD_NUMBER: builtins.int
    OUTPUT_COUNT_FIELD_NUMBER: builtins.int
    name: builtins.str
    node_id: builtins.int
    type_name: builtins.str
    soc_op_id: builtins.int
    padding_id: builtins.int
    input_count: builtins.int
    output_count: builtins.int
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        node_id: builtins.int | None = ...,
        type_name: builtins.str | None = ...,
        soc_op_id: builtins.int | None = ...,
        padding_id: builtins.int | None = ...,
        input_count: builtins.int | None = ...,
        output_count: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "input_count",
            b"input_count",
            "name",
            b"name",
            "node_id",
            b"node_id",
            "output_count",
            b"output_count",
            "padding_id",
            b"padding_id",
            "soc_op_id",
            b"soc_op_id",
            "type_name",
            b"type_name",
        ],
    ) -> None: ...

global___GraphTransferNodeInfo = GraphTransferNodeInfo

@typing.final
class GraphTransferConstNodeInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    NODE_ID_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    node_id: builtins.int
    data: builtins.bytes
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    @property
    def shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        node_id: builtins.int | None = ...,
        shape: collections.abc.Iterable[builtins.int] | None = ...,
        data: builtins.bytes | None = ...,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal["data", b"data", "dtype", b"dtype", "name", b"name", "node_id", b"node_id", "shape", b"shape"],
    ) -> None: ...

global___GraphTransferConstNodeInfo = GraphTransferConstNodeInfo

@typing.final
class GraphTransferNodeInputInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_ID_FIELD_NUMBER: builtins.int
    NODE_INPUT_FIELD_NUMBER: builtins.int
    node_id: builtins.int
    @property
    def node_input(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphTransferNodeInput]: ...
    def __init__(
        self,
        *,
        node_id: builtins.int | None = ...,
        node_input: collections.abc.Iterable[global___GraphTransferNodeInput] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["node_id", b"node_id", "node_input", b"node_input"]) -> None: ...

global___GraphTransferNodeInputInfo = GraphTransferNodeInputInfo

@typing.final
class GraphTransferNodeOutputInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_ID_FIELD_NUMBER: builtins.int
    MAX_BYTE_SIZE_FIELD_NUMBER: builtins.int
    node_id: builtins.int
    @property
    def max_byte_size(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self, *, node_id: builtins.int | None = ..., max_byte_size: collections.abc.Iterable[builtins.int] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["max_byte_size", b"max_byte_size", "node_id", b"node_id"]) -> None: ...

global___GraphTransferNodeOutputInfo = GraphTransferNodeOutputInfo

@typing.final
class GraphTransferGraphInputNodeInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    @property
    def shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        shape: collections.abc.Iterable[builtins.int] | None = ...,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dtype", b"dtype", "name", b"name", "shape", b"shape"]) -> None: ...

global___GraphTransferGraphInputNodeInfo = GraphTransferGraphInputNodeInfo

@typing.final
class GraphTransferGraphOutputNodeInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    @property
    def shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        shape: collections.abc.Iterable[builtins.int] | None = ...,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dtype", b"dtype", "name", b"name", "shape", b"shape"]) -> None: ...

global___GraphTransferGraphOutputNodeInfo = GraphTransferGraphOutputNodeInfo

@typing.final
class GraphTransferInfo(google.protobuf.message.Message):
    """Protocol buffer representing a handle to a tensorflow resource. Handles are
    not valid across executions, but can be serialized back and forth from within
    a single run.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Destination:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _DestinationEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[GraphTransferInfo._Destination.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        NOP: GraphTransferInfo._Destination.ValueType  # 0
        HEXAGON: GraphTransferInfo._Destination.ValueType  # 1

    class Destination(_Destination, metaclass=_DestinationEnumTypeWrapper): ...
    NOP: GraphTransferInfo.Destination.ValueType  # 0
    HEXAGON: GraphTransferInfo.Destination.ValueType  # 1

    NODE_INFO_FIELD_NUMBER: builtins.int
    CONST_NODE_INFO_FIELD_NUMBER: builtins.int
    NODE_INPUT_INFO_FIELD_NUMBER: builtins.int
    NODE_OUTPUT_INFO_FIELD_NUMBER: builtins.int
    GRAPH_INPUT_NODE_INFO_FIELD_NUMBER: builtins.int
    GRAPH_OUTPUT_NODE_INFO_FIELD_NUMBER: builtins.int
    DESTINATION_FIELD_NUMBER: builtins.int
    destination: global___GraphTransferInfo.Destination.ValueType
    """Destination of graph transfer"""
    @property
    def node_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphTransferNodeInfo]: ...
    @property
    def const_node_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphTransferConstNodeInfo]: ...
    @property
    def node_input_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphTransferNodeInputInfo]: ...
    @property
    def node_output_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphTransferNodeOutputInfo]: ...
    @property
    def graph_input_node_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphTransferGraphInputNodeInfo]:
        """Input Node parameters of transferred graph"""

    @property
    def graph_output_node_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GraphTransferGraphOutputNodeInfo]: ...
    def __init__(
        self,
        *,
        node_info: collections.abc.Iterable[global___GraphTransferNodeInfo] | None = ...,
        const_node_info: collections.abc.Iterable[global___GraphTransferConstNodeInfo] | None = ...,
        node_input_info: collections.abc.Iterable[global___GraphTransferNodeInputInfo] | None = ...,
        node_output_info: collections.abc.Iterable[global___GraphTransferNodeOutputInfo] | None = ...,
        graph_input_node_info: collections.abc.Iterable[global___GraphTransferGraphInputNodeInfo] | None = ...,
        graph_output_node_info: collections.abc.Iterable[global___GraphTransferGraphOutputNodeInfo] | None = ...,
        destination: global___GraphTransferInfo.Destination.ValueType | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "const_node_info",
            b"const_node_info",
            "destination",
            b"destination",
            "graph_input_node_info",
            b"graph_input_node_info",
            "graph_output_node_info",
            b"graph_output_node_info",
            "node_info",
            b"node_info",
            "node_input_info",
            b"node_input_info",
            "node_output_info",
            b"node_output_info",
        ],
    ) -> None: ...

global___GraphTransferInfo = GraphTransferInfo



"""Jac meta path importer.

This module implements PEP 451-compliant import hooks for .jac modules.
It leverages Python's modern import machinery (importlib.abc) to seamlessly
integrate Jac modules into Python's import system.
"""

from __future__ import annotations

import importlib.abc
import importlib.machinery
import importlib.util
import os
from collections.abc import Sequence
from functools import cache
from pathlib import Path
from types import ModuleType

from jaclang.pycore.log import logging
from jaclang.pycore.modresolver import (
    get_jac_search_paths,
)

logger = logging.getLogger(__name__)


@cache
def _discover_minimal_compile_modules() -> frozenset[str]:
    """Auto-discover .jac compiler passes that need minimal compilation."""
    jaclang_dir = Path(__file__).parent
    passes_dir = jaclang_dir / "compiler" / "passes"
    modules = set()

    for subdir in ["main", "ecmascript"]:
        for jac_file in (passes_dir / subdir).rglob("*.jac"):
            if jac_file.name.endswith(".impl.jac"):
                continue
            module_path = jac_file.relative_to(jaclang_dir).with_suffix("")
            modules.add(f"jaclang.{module_path.as_posix().replace('/', '.')}")

    return frozenset(modules)


class JacMetaImporter(importlib.abc.MetaPathFinder, importlib.abc.Loader):
    """Meta path importer to load .jac modules via Python's import system."""

    @property
    def MINIMAL_COMPILE_MODULES(self) -> frozenset[str]:  # noqa: N802
        """Compiler passes written in Jac that need minimal compilation."""
        return _discover_minimal_compile_modules()

    def find_spec(
        self,
        fullname: str,
        path: Sequence[str] | None = None,
        target: ModuleType | None = None,
    ) -> importlib.machinery.ModuleSpec | None:
        """Find the spec for the module."""
        if path is None:
            # Top-level import
            paths_to_search = get_jac_search_paths()
            module_path_parts = fullname.split(".")
        else:
            # Submodule import
            paths_to_search = [*path]
            module_path_parts = fullname.split(".")[-1:]

        for search_path in paths_to_search:
            candidate_path = os.path.join(search_path, *module_path_parts)
            # Check for directory package
            if os.path.isdir(candidate_path):
                init_file = os.path.join(candidate_path, "__init__.jac")
                if os.path.isfile(init_file):
                    return importlib.util.spec_from_file_location(
                        fullname,
                        init_file,
                        loader=self,
                        submodule_search_locations=[candidate_path],
                    )
                init_sv_file = os.path.join(candidate_path, "__init__.sv.jac")
                if os.path.isfile(init_sv_file):
                    return importlib.util.spec_from_file_location(
                        fullname,
                        init_sv_file,
                        loader=self,
                        submodule_search_locations=[candidate_path],
                    )
                init_cl_file = os.path.join(candidate_path, "__init__.cl.jac")
                if os.path.isfile(init_cl_file):
                    return importlib.util.spec_from_file_location(
                        fullname,
                        init_cl_file,
                        loader=self,
                        submodule_search_locations=[candidate_path],
                    )
            # Check for .jac file
            jac_file = candidate_path + ".jac"
            if os.path.isfile(jac_file):
                return importlib.util.spec_from_file_location(
                    fullname, jac_file, loader=self
                )
            # Check for .sv.jac file (server-side explicit)
            sv_jac_file = candidate_path + ".sv.jac"
            if os.path.isfile(sv_jac_file):
                return importlib.util.spec_from_file_location(
                    fullname, sv_jac_file, loader=self
                )
            # Check for .cl.jac file (client-side)
            cl_jac_file = candidate_path + ".cl.jac"
            if os.path.isfile(cl_jac_file):
                return importlib.util.spec_from_file_location(
                    fullname, cl_jac_file, loader=self
                )

        return None

    def create_module(self, spec: importlib.machinery.ModuleSpec) -> ModuleType | None:
        """Create the module."""
        return None  # use default machinery

    def exec_module(self, module: ModuleType) -> None:
        """Execute the module by loading and executing its bytecode.

        This method implements PEP 451's exec_module() protocol, which separates
        module creation from execution. It handles both package (__init__.jac) and
        regular module (.jac/.py) execution.
        """
        from jaclang.pycore.runtime import JacRuntime as Jac

        if not module.__spec__ or not module.__spec__.origin:
            raise ImportError(
                f"Cannot find spec or origin for module {module.__name__}"
            )

        file_path = module.__spec__.origin
        is_pkg = module.__spec__.submodule_search_locations is not None

        # Register module in JacRuntime's tracking (skip internal jaclang modules)
        if not module.__name__.startswith("jaclang."):
            Jac.load_module(module.__name__, module)

        # Use minimal compilation for compiler passes to avoid circular imports
        use_minimal = module.__name__ in self.MINIMAL_COMPILE_MODULES

        # Get and execute bytecode using the compiler singleton
        codeobj = Jac.get_compiler().get_bytecode(
            full_target=file_path,
            target_program=Jac.get_program(),
            minimal=use_minimal,
        )
        if not codeobj:
            if is_pkg:
                # Empty package is OK - just register it
                return
            raise ImportError(f"No bytecode found for {file_path}")
        # Execute the bytecode directly in the module's namespace
        exec(codeobj, module.__dict__)

    def get_code(self, fullname: str) -> object | None:
        """Get the code object for a module.

        This method is required by runpy when using `python -m module`.
        """
        from jaclang.pycore.runtime import JacRuntime as Jac

        # Find the .jac file for this module
        paths_to_search = get_jac_search_paths()
        module_path_parts = fullname.split(".")

        # Use minimal compilation for compiler passes to avoid circular imports
        use_minimal = fullname in self.MINIMAL_COMPILE_MODULES

        compiler = Jac.get_compiler()
        program = Jac.get_program()

        for search_path in paths_to_search:
            candidate_path = os.path.join(search_path, *module_path_parts)
            # Check for directory package
            if os.path.isdir(candidate_path):
                init_file = os.path.join(candidate_path, "__init__.jac")
                if os.path.isfile(init_file):
                    return compiler.get_bytecode(
                        full_target=init_file,
                        target_program=program,
                        minimal=use_minimal,
                    )
                init_cl_file = os.path.join(candidate_path, "__init__.cl.jac")
                if os.path.isfile(init_cl_file):
                    return compiler.get_bytecode(
                        full_target=init_cl_file,
                        target_program=program,
                        minimal=use_minimal,
                    )
            # Check for .jac file
            jac_file = candidate_path + ".jac"
            if os.path.isfile(jac_file):
                return compiler.get_bytecode(
                    full_target=jac_file,
                    target_program=program,
                    minimal=use_minimal,
                )
            cl_jac_file = candidate_path + ".cl.jac"
            if os.path.isfile(cl_jac_file):
                return compiler.get_bytecode(
                    full_target=cl_jac_file,
                    target_program=program,
                    minimal=use_minimal,
                )

        return None



from networkx.utils.backends import _dispatchable

__all__ = [
    "LCF_graph",
    "bull_graph",
    "chvatal_graph",
    "cubical_graph",
    "desargues_graph",
    "diamond_graph",
    "dodecahedral_graph",
    "frucht_graph",
    "heawood_graph",
    "hoffman_singleton_graph",
    "house_graph",
    "house_x_graph",
    "icosahedral_graph",
    "krackhardt_kite_graph",
    "moebius_kantor_graph",
    "octahedral_graph",
    "pappus_graph",
    "petersen_graph",
    "sedgewick_maze_graph",
    "tetrahedral_graph",
    "truncated_cube_graph",
    "truncated_tetrahedron_graph",
    "tutte_graph",
]

@_dispatchable
def LCF_graph(n, shift_list, repeats, create_using=None): ...
@_dispatchable
def bull_graph(create_using=None): ...
@_dispatchable
def chvatal_graph(create_using=None): ...
@_dispatchable
def cubical_graph(create_using=None): ...
@_dispatchable
def desargues_graph(create_using=None): ...
@_dispatchable
def diamond_graph(create_using=None): ...
@_dispatchable
def dodecahedral_graph(create_using=None): ...
@_dispatchable
def frucht_graph(create_using=None): ...
@_dispatchable
def heawood_graph(create_using=None): ...
@_dispatchable
def hoffman_singleton_graph(): ...
@_dispatchable
def house_graph(create_using=None): ...
@_dispatchable
def house_x_graph(create_using=None): ...
@_dispatchable
def icosahedral_graph(create_using=None): ...
@_dispatchable
def krackhardt_kite_graph(create_using=None): ...
@_dispatchable
def moebius_kantor_graph(create_using=None): ...
@_dispatchable
def octahedral_graph(create_using=None): ...
@_dispatchable
def pappus_graph(): ...
@_dispatchable
def petersen_graph(create_using=None): ...
@_dispatchable
def sedgewick_maze_graph(create_using=None): ...
@_dispatchable
def tetrahedral_graph(create_using=None): ...
@_dispatchable
def truncated_cube_graph(create_using=None): ...
@_dispatchable
def truncated_tetrahedron_graph(create_using=None): ...
@_dispatchable
def tutte_graph(create_using=None): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatchable

from ..classes import MultiDiGraph

__all__ = ["gn_graph", "gnc_graph", "gnr_graph", "random_k_out_graph", "scale_free_graph"]

@_dispatchable
def gn_graph(n, kernel=None, create_using=None, seed=None): ...
@_dispatchable
def gnr_graph(n, p, create_using=None, seed=None): ...
@_dispatchable
def gnc_graph(n, create_using=None, seed=None): ...
@_dispatchable
def scale_free_graph(
    n,
    alpha: float = 0.41,
    beta: float = 0.54,
    gamma: float = 0.05,
    delta_in: float = 0.2,
    delta_out: float = 0,
    create_using=None,
    seed=None,
    initial_graph=None,
): ...
@_dispatchable
def random_uniform_k_out_graph(n: int, k: int, self_loops: bool = True, with_replacement: bool = True, seed=None): ...
@_dispatchable
def random_k_out_graph(n, k, alpha, self_loops: bool = True, seed=None) -> MultiDiGraph[Incomplete]: ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import (
    # pyarrow types returned as Any to avoid depending on pyarrow (40 MB) in stubs
    Any as _PAArray,
    Any as _PAField,
    Any as _PATable,
    Literal,
    Protocol,
    type_check_only,
)
from typing_extensions import CapsuleType, TypeAlias

import numpy as np
from numpy.typing import NDArray

from ..array import GeometryArray
from ..geodataframe import GeoDataFrame

# Literal for language server completions and str because runtime normalizes to lowercase
_GeomEncoding: TypeAlias = Literal["WKB", "geoarrow"] | str  # noqa: Y051

@type_check_only
class _PyarrowTableLike(Protocol):
    def __arrow_c_stream__(self, requested_schema=None) -> CapsuleType: ...

@type_check_only
class _PyarrowFieldLike(Protocol):
    def __arrow_c_schema__(self) -> CapsuleType: ...

@type_check_only
class _PyarrowArrayLike(Protocol):
    def __arrow_c_array__(self) -> tuple[CapsuleType, CapsuleType]: ...

GEOARROW_ENCODINGS: list[str]

class ArrowTable:
    def __init__(self, pa_table: _PyarrowTableLike) -> None: ...
    def __arrow_c_stream__(self, requested_schema=None) -> CapsuleType: ...

class GeoArrowArray:
    def __init__(self, pa_field: _PyarrowFieldLike, pa_array: _PyarrowArrayLike) -> None: ...
    def __arrow_c_array__(self, requested_schema=None) -> tuple[CapsuleType, CapsuleType]: ...

def geopandas_to_arrow(
    df: GeoDataFrame,
    index: bool | None = None,
    geometry_encoding: _GeomEncoding = "WKB",
    interleaved: bool = True,
    include_z: bool | None = None,
) -> tuple[_PATable, dict[str, str]]: ...
def construct_wkb_array(
    shapely_arr: NDArray[np.object_], *, field_name: str = "geometry", crs: str | None = None
) -> tuple[_PAField, _PAArray]: ...
def construct_geometry_array(
    shapely_arr: NDArray[np.object_],
    include_z: bool | None = None,
    *,
    field_name: str = "geometry",
    crs: str | None = None,
    interleaved: bool = True,
) -> tuple[_PAField, _PAArray]: ...
def arrow_to_geopandas(
    table, geometry: str | None = None, to_pandas_kwargs: Mapping[str, Incomplete] | None = None
) -> GeoDataFrame: ...
def arrow_to_geometry_array(arr) -> GeometryArray: ...
def construct_shapely_array(arr: _PAArray, extension_name: str) -> NDArray[np.object_]: ...



from collections.abc import Callable, Mapping
from typing_extensions import TypeAlias

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

_WeightFunction: TypeAlias = Callable[..., int]

__all__ = [
    "eccentricity",
    "diameter",
    "harmonic_diameter",
    "radius",
    "periphery",
    "center",
    "barycenter",
    "resistance_distance",
    "kemeny_constant",
    "effective_graph_resistance",
]

@_dispatchable
def eccentricity(
    G: Graph[_Node],
    v: _Node | None = None,
    sp: Mapping[_Node, Mapping[_Node, int]] | None = None,
    weight: str | _WeightFunction | None = None,
) -> int | dict[_Node, int]: ...  # TODO: overload on v: dict if v is None else int
@_dispatchable
def diameter(
    G: Graph[_Node], e: Mapping[_Node, int] | None = None, usebounds: bool = False, weight: str | _WeightFunction | None = None
) -> int: ...
@_dispatchable
def harmonic_diameter(
    G: Graph[_Node], sp: Mapping[_Node, Mapping[_Node, int]] | None = None, *, weight: str | _WeightFunction | None = None
) -> float: ...
@_dispatchable
def periphery(
    G: Graph[_Node], e: Mapping[_Node, int] | None = None, usebounds: bool = False, weight: str | _WeightFunction | None = None
) -> list[_Node]: ...
@_dispatchable
def radius(
    G: Graph[_Node], e: Mapping[_Node, int] | None = None, usebounds: bool = False, weight: str | _WeightFunction | None = None
) -> int: ...
@_dispatchable
def center(
    G: Graph[_Node], e: Mapping[_Node, int] | None = None, usebounds: bool = False, weight: str | _WeightFunction | None = None
) -> list[_Node]: ...
@_dispatchable
def barycenter(
    G: Graph[_Node],
    weight: str | _WeightFunction | None = None,
    attr: str | None = None,
    sp: Mapping[_Node, Mapping[_Node, int]] | None = None,
) -> list[_Node]: ...
@_dispatchable
def resistance_distance(
    G: Graph[_Node], nodeA: _Node | None = None, nodeB: _Node | None = None, weight: str | None = None, invert_weight: bool = True
) -> float | dict[_Node, float]: ...  # TODO: overload on the nodes: float if both are specified else dict
@_dispatchable
def effective_graph_resistance(G: Graph[_Node], weight: str | None = None, invert_weight: bool = True) -> float: ...
@_dispatchable
def kemeny_constant(G: Graph[_Node], *, weight: str | None = None) -> float: ...



from _typeshed import Unused
from typing import Any

from django.db.models import Model, QuerySet
from django.http import HttpRequest, HttpResponse
from django.views.generic import View
from django.views.generic.list import MultipleObjectMixin, MultipleObjectTemplateResponseMixin

from .constants import ALL_FIELDS
from .filterset import FilterSet

class FilterMixin:
    filterset_class: type[FilterSet] | None
    filterset_fields = ALL_FIELDS
    strict: bool
    def get_filterset_class(self) -> type[FilterSet] | None: ...
    def get_filterset(self, filterset_class: type[FilterSet]) -> FilterSet: ...
    def get_filterset_kwargs(self, filterset_class: type[FilterSet]) -> dict[str, Any]: ...  # Filterset init params vary
    def get_strict(self) -> bool: ...

class BaseFilterView(FilterMixin, MultipleObjectMixin[Any], View):  # Generic model type
    filterset: FilterSet
    object_list: QuerySet[Any]  # Filtered objects of any model type

    def get(self, request: HttpRequest, *args: Unused, **kwargs: Unused) -> HttpResponse: ...

class FilterView(MultipleObjectTemplateResponseMixin, BaseFilterView):
    template_name_suffix: str

def object_filter(
    request: HttpRequest,
    model: type[Model] | None = None,
    queryset: QuerySet[Any] | None = None,  # Base queryset for any model
    template_name: str | None = None,
    extra_context: dict[str, Any] | None = None,  # Template context values vary
    context_processors: list[Any] | None = None,  # Context processors vary by implementation
    filter_class: type[FilterSet] | None = None,
) -> HttpResponse: ...
